@@ -1,16 +1,22 @@
 /**************************************************************************
  Copyright:
-      (C) 2008 - 2009  Alexander Shaduri <ashaduri 'at' gmail.com>
+      (C) 2008 - 2012  Alexander Shaduri <ashaduri 'at' gmail.com>
  License: See LICENSE_gsmartcontrol.txt
 ***************************************************************************/
+/// \file
+/// \author Alexander Shaduri
+/// \ingroup applib
+/// \weakgroup applib
+/// @{
 
 #ifndef APP_GTKMM_FEATURES_H
 #define APP_GTKMM_FEATURES_H
 
-#include <gtkmmconfig.h>
+#include <gtkmm.h>
 
 
-// Similar to GTK_CHECK_VERSION, but for gtkmm (this should have been in it)
+/// \def APP_GTKMM_CHECK_VERSION(major, minor, micro)
+/// Similar to GTK_CHECK_VERSION, but for gtkmm, which lacks this for some reason.
 #ifndef APP_GTKMM_CHECK_VERSION
 	#define APP_GTKMM_CHECK_VERSION(major, minor, micro) \
 		(GTKMM_MAJOR_VERSION > (major) \
@@ -23,13 +29,6 @@
 
 
 
-// New tooltip API is available since 2.12
-#if !APP_GTKMM_CHECK_VERSION(2, 12, 0)
-	#define APP_GTKMM_OLD_TOOLTIPS 1
-#endif
-
-
-
 
 // If default virtual on_* members are disabled in gtkmm, use this in constructors
 // to connect on_signal_name methods.
@@ -51,14 +50,13 @@
 */
 
 
-// Connect to a signal _before_ the default handler. That is, if you want
-// to have, say, on_delete_event() in your window-inherited class, define
-// on_delete_event_before() instead and return true (handled) from it
-// if it's X event handler.
-// This method will work regardless of presence of default virtual handlers
-// in parent class. It will also avoid calling the handler twice (one from
-// signal, another from default virtual handler).
-
+/// Connect to a signal _before_ the default handler. That is, if you want
+/// to have, say, on_delete_event() in your window-inherited class, define
+/// on_delete_event_before() instead and return true (handled) from it
+/// if it's X event handler.
+/// This method will work regardless of presence of default virtual handlers
+/// in parent class. It will also avoid calling the handler twice (one from
+/// signal, another from default virtual handler).
 #define APP_GTKMM_CONNECT_VIRTUAL(signal_name) \
 	this->signal_ ## signal_name ().connect(sigc::mem_fun(*this, &self_type::on_ ## signal_name ## _before), false)
 
@@ -68,3 +66,5 @@
 
 
 #endif
+
+/// @}
