@@ -1,8 +1,13 @@
 /**************************************************************************
  Copyright:
-      (C) 2008 - 2009  Alexander Shaduri <ashaduri 'at' gmail.com>
+      (C) 2008 - 2012  Alexander Shaduri <ashaduri 'at' gmail.com>
  License: See LICENSE_gsmartcontrol.txt
 ***************************************************************************/
+/// \file
+/// \author Alexander Shaduri
+/// \ingroup applib
+/// \weakgroup applib
+/// @{
 
 #include <sys/types.h>
 #include <cerrno>  // errno (not std::errno, it may be a macro)
@@ -37,25 +42,28 @@ extern "C" {
 
 	// callbacks
 
-
+	/// Child process watcher callback
 	inline void cmdex_child_watch_handler(GPid arg_pid, int waitpid_status, gpointer data)
 	{
 		Cmdex::on_child_watch_handler(arg_pid, waitpid_status, data);
 	}
 
 
+	/// Child process stdout handler callback
 	inline gboolean cmdex_on_channel_io_stdout(GIOChannel* source, GIOCondition cond, gpointer data)
 	{
 		return Cmdex::on_channel_io(source, cond, static_cast<Cmdex*>(data), Cmdex::channel_type_stdout);
 	}
 
+
+	/// Child process stderr handler callback
 	inline gboolean cmdex_on_channel_io_stderr(GIOChannel* source, GIOCondition cond, gpointer data)
 	{
 		return Cmdex::on_channel_io(source, cond, static_cast<Cmdex*>(data), Cmdex::channel_type_stderr);
 	}
 
 
-
+	/// Child process termination timeout handler
 	inline gboolean cmdex_on_term_timeout(gpointer data)
 	{
 		DBG_FUNCTION_ENTER_MSG;
@@ -65,11 +73,12 @@ extern "C" {
 	}
 
 
+	/// Child process kill timeout handler
 	inline gboolean cmdex_on_kill_timeout(gpointer data)
 	{
 		DBG_FUNCTION_ENTER_MSG;
 		Cmdex* self = static_cast<Cmdex*>(data);
-		self->try_stop(hz::SIGNAL_SIGTERM);
+		self->try_stop(hz::SIGNAL_SIGKILL);
 		return false;  // one-time call
 	}
 
@@ -83,8 +92,10 @@ extern "C" {
 
 bool Cmdex::execute()
 {
-	if (this->running_ || this->stopped_cleanup_needed())
+	DBG_FUNCTION_ENTER_MSG;
+	if (this->running_ || this->stopped_cleanup_needed()) {
 		return false;
+	}
 
 	cleanup_members();
 	clear_errors();
@@ -120,10 +131,10 @@ bool Cmdex::execute()
 	#endif
 
 	hz::ScopedEnv lang_env("LANG", "C", change_lang);
-
+	hz::ScopedEnv lc_numeric_env("LC_NUMERIC", "C", change_lang);
 
 	debug_out_info("app", DBG_FUNC_MSG << "Executing \"" << cmd << "\".\n");
-
+/*
 	if (argvp) {
 		debug_out_dump("app", DBG_FUNC_MSG << "Dumping argvp:\n");
 		gchar** elem = argvp.get();
@@ -132,7 +143,7 @@ bool Cmdex::execute()
 			++elem;
 		}
 	}
-
+*/
 
 	// Execute the command
 
@@ -216,9 +227,11 @@ bool Cmdex::execute()
 
 	this->event_source_id_stdout_ = g_io_add_watch_full(channel_stdout_, io_priority, cond,
 			&cmdex_on_channel_io_stdout, this, NULL);
+// 	g_io_channel_unref(channel_stdout_);  // g_io_add_watch_full() holds its own reference
 
 	this->event_source_id_stderr_ = g_io_add_watch_full(channel_stderr_, io_priority, cond,
 			&cmdex_on_channel_io_stderr, this, NULL);
+// 	g_io_channel_unref(channel_stderr_);  // g_io_add_watch_full() holds its own reference
 
 
 	// If using SPAWN_DO_NOT_REAP_CHILD, this is needed to avoid zombies.
@@ -229,6 +242,7 @@ bool Cmdex::execute()
 
 	this->running_ = true;  // the process is running now.
 
+	DBG_FUNCTION_EXIT_MSG;
 	return true;
 }
 
@@ -238,8 +252,14 @@ bool Cmdex::execute()
 // send SIGTERM(15) (terminate)
 bool Cmdex::try_stop(hz::signal_t sig)
 {
+	DBG_FUNCTION_ENTER_MSG;
+#ifdef _WIN32
+	if (!this->running_ || this->pid_ == 0)
+		return false;
+#else
 	if (!this->running_ || this->pid_ <= 0)
 		return false;
+#endif
 
 	// other variants: SIGHUP(1) (terminal closed), SIGINT(2) (Ctrl-C),
 	// SIGKILL(9) (kill).
@@ -253,14 +273,23 @@ bool Cmdex::try_stop(hz::signal_t sig)
 	// Possible: EPERM (no permissions), ESRCH (no such process, or zombie)
 	push_error(Error<int>("errno", ErrorLevel::error, errno), false);
 
+	DBG_FUNCTION_EXIT_MSG;
 	return false;
 }
 
 
 
+bool Cmdex::try_kill()
+{
+	DBG_TRACE_POINT_AUTO;
+	return try_stop(hz::SIGNAL_SIGKILL);
+}
+
+
 
 void Cmdex::set_stop_timeouts(int term_timeout_msec, int kill_timeout_msec)
 {
+	DBG_FUNCTION_ENTER_MSG;
 	DBG_ASSERT(term_timeout_msec == 0 || kill_timeout_msec == 0 || kill_timeout_msec > term_timeout_msec);
 
 	if (!this->running_)  // process not running
@@ -273,6 +302,8 @@ void Cmdex::set_stop_timeouts(int term_timeout_msec, int kill_timeout_msec)
 
 	if (kill_timeout_msec != 0)
 		event_source_id_kill = g_timeout_add(kill_timeout_msec, &cmdex_on_kill_timeout, this);
+
+	DBG_FUNCTION_EXIT_MSG;
 }
 
 
@@ -280,6 +311,7 @@ void Cmdex::set_stop_timeouts(int term_timeout_msec, int kill_timeout_msec)
 
 void Cmdex::unset_stop_timeouts()
 {
+	DBG_FUNCTION_ENTER_MSG;
 	if (event_source_id_term) {
 		GSource* source_term = g_main_context_find_source_by_id(NULL, event_source_id_term);
 		if (source_term)
@@ -293,6 +325,7 @@ void Cmdex::unset_stop_timeouts()
 			g_source_destroy(source_kill);
 		event_source_id_kill = 0;
 	}
+	DBG_FUNCTION_EXIT_MSG;
 }
 
 
@@ -300,8 +333,7 @@ void Cmdex::unset_stop_timeouts()
 // executed in main thread, manually by the caller.
 void Cmdex::stopped_cleanup()
 {
-// 	DBG_FUNCTION_ENTER_MSG;
-
+	DBG_FUNCTION_ENTER_MSG;
 	if (this->running_ || !this->stopped_cleanup_needed())  // huh?
 		return;
 
@@ -338,6 +370,7 @@ void Cmdex::stopped_cleanup()
 	cleanup_members();
 
 	this->running_ = false;
+	DBG_FUNCTION_EXIT_MSG;
 }
 
 
@@ -356,6 +389,25 @@ void Cmdex::on_child_watch_handler(GPid arg_pid, int waitpid_status, gpointer da
 	self->child_watch_handler_called_ = true;
 	self->running_ = false;  // process is not running anymore
 
+	// These are needed because Windows doesn't read the remaining data otherwise.
+	g_io_channel_flush(self->channel_stdout_, NULL);
+	on_channel_io(self->channel_stdout_, GIOCondition(0), self, channel_type_stdout);
+
+	g_io_channel_flush(self->channel_stderr_, NULL);
+	on_channel_io(self->channel_stderr_, GIOCondition(0), self, channel_type_stderr);
+
+	if (self->channel_stdout_) {
+		g_io_channel_shutdown(self->channel_stdout_, false, NULL);
+		g_io_channel_unref(self->channel_stdout_);
+		self->channel_stdout_ = 0;
+	}
+
+	if (self->channel_stderr_) {
+		g_io_channel_shutdown(self->channel_stderr_, false, NULL);
+		g_io_channel_unref(self->channel_stderr_);
+		self->channel_stderr_ = 0;
+	}
+
 	// Remove fd IO callbacks. They may actually be removed already (note sure about this).
 	// This will force calling the iochannel callback (they may not be called
 	// otherwise at all if there was no output).
@@ -386,9 +438,10 @@ void Cmdex::on_child_watch_handler(GPid arg_pid, int waitpid_status, gpointer da
 
 
 
-gboolean Cmdex::on_channel_io(GIOChannel* source,
+gboolean Cmdex::on_channel_io(GIOChannel* channel,
 		GIOCondition cond, Cmdex* self, channel_t type)
 {
+// 	DBG_FUNCTION_ENTER_MSG;
 // 	debug_out_dump("app", "Cmdex::on_channel_io("
 // 			<< (type == channel_type_stdout ? "STDOUT" : "STDERR") << ") " << int(cond) << "\n");
 
@@ -405,13 +458,10 @@ gboolean Cmdex::on_channel_io(GIOChannel* source,
 	const gsize count = 1;
 	gchar buf[count] = {0};
 
-	GIOChannel* channel = 0;
 	std::string* output_str = 0;
 	if (type == channel_type_stdout) {
-		channel = self->channel_stdout_;
 		output_str = &self->str_stdout_;
 	} else if (type == channel_type_stderr) {
-		channel = self->channel_stderr_;
 		output_str = &self->str_stderr_;
 	}
 
@@ -444,7 +494,21 @@ gboolean Cmdex::on_channel_io(GIOChannel* source,
 
 
 
+void Cmdex::cleanup_members()
+{
+	kill_signal_sent_ = 0;
+	child_watch_handler_called_ = false;
+	pid_ = 0;
+	waitpid_status_ = 0;
+	event_source_id_stdout_ = 0;
+	event_source_id_stderr_ = 0;
+	fd_stdout_ = 0;
+	fd_stderr_ = 0;
+}
 
 
 
 
+
+
+/// @}
