@@ -1,18 +1,20 @@
 /**************************************************************************
  Copyright:
-      (C) 2008 - 2009  Alexander Shaduri <ashaduri 'at' gmail.com>
+      (C) 2008 - 2012  Alexander Shaduri <ashaduri 'at' gmail.com>
  License: See LICENSE_zlib.txt file
 ***************************************************************************/
 
 #ifndef GLOBAL_MACROS_H
 #define GLOBAL_MACROS_H
 
-/*
+/**
+\file
 This file serves as a compile-time configuration for various
 library components.
 
-This file is not included directly, but through the compiler option
--include global_macros.h
+This file is included from hz_config.h.
+Additionally, it may be included through compiler's "-include" option
+(if supported) for pch support.
 */
 
 
@@ -22,8 +24,8 @@ This file is not included directly, but through the compiler option
 #endif
 
 
-// So that others may check if "-include" works
-#define APP_GLOBAL_MACROS_INCLUDED
+/// So that others may check whether this file was included or not.
+#define APP_GLOBAL_MACROS_INCLUDED 1
 
 
 // #ifdef _WIN32
@@ -31,77 +33,8 @@ This file is not included directly, but through the compiler option
 // 	#define WINVER 0x0501
 // #endif
 
-
-// List most of the included system headers here - they will
-// be precompiled together with this file.
-
-#ifdef ENABLE_PCH
-
-	#ifdef HAVE_STDC___H
-		// All libstdc++ headers
-		#include <stdc++.h>
-
-	#else
-		#include <string>
-
-		#include <vector>
-		#include <list>
-		#include <stack>
-		#include <map>
-
-		#include <iosfwd>
-		#include <sstream>
-		#include <iomanip>
-		#include <iostream>
-		#include <ios>
-
-		#include <limits>
-		#include <locale>
-		#include <exception>
-		#include <stdexcept>
-
-		#include <cctype>
-		#include <cstdlib>
-		#include <cstdio>
-		#include <cassert>
-		#include <csignal>
-		#include <cerrno>
-		#include <cstddef>
-		#include <cstring>
-
-		#include <cxxabi.h>  // is this included in stdc++.h?
-	#endif
-
-	// most of the other headers
-	#include <sys/types.h>
-
-// 	#include <dirent.h>
-
-	#if defined ENABLE_GLIB && ENABLE_GLIB
-		#include <glib.h>
-	#endif
-	#if defined ENABLE_GLIBMM && ENABLE_GLIBMM
-		#include <sigc++/sigc++.h>
-		#include <gtkmm.h>  // if we have glibmm, we're building everything
-		#include <gtk/gtk.h>
-		#include <glibmm.h>
-	#endif
-
-	#ifdef _WIN32
-		#include <windows.h>
-	#endif
-
-#endif  // pch
-
-
-
 // this is either defined to 1 (by autoconf), or undefined
 #ifdef HAVE_LONG_LONG_INT
-	// pcrecpp uses this
-	#ifndef HAVE_LONG_LONG
-		#define HAVE_LONG_LONG
-	#endif
-
 	#define DISABLE_LL_INT 0
 
 #else  // there is no long long int
@@ -113,11 +46,6 @@ This file is not included directly, but through the compiler option
 
 // this is either defined to 1 (by autoconf), or undefined
 #ifdef HAVE_UNSIGNED_LONG_LONG_INT
-	// pcrecpp uses this
-	#ifndef HAVE_UNSIGNED_LONG_LONG
-		#define HAVE_UNSIGNED_LONG_LONG
-	#endif
-
 	#define DISABLE_ULL_INT 0
 
 #else  // there is no long long int
