@@ -1,16 +1,22 @@
 /**************************************************************************
  Copyright:
-      (C) 2008 - 2009  Alexander Shaduri <ashaduri 'at' gmail.com>
+      (C) 2008 - 2012  Alexander Shaduri <ashaduri 'at' gmail.com>
  License: See LICENSE_gsmartcontrol.txt
 ***************************************************************************/
+/// \file
+/// \author Alexander Shaduri
+/// \ingroup gsc
+/// \weakgroup gsc
+/// @{
 
 #include <gtkmm.h>
-#include <algorithm>  // std::sort
 #include <vector>
 
 #include "hz/string_algo.h"  // string_split
 #include "hz/fs_file.h"  // hz::File
 #include "hz/debug.h"
+#include "hz/scoped_ptr.h"
+#include "hz/launch_url.h"
 #include "rconfig/rconfig_mini.h"
 #include "applib/storage_detector.h"
 #include "applib/smartctl_parser.h"
@@ -24,13 +30,14 @@
 #include "gsc_init.h"  // app_quit()
 #include "gsc_about_dialog.h"
 #include "gsc_info_window.h"
-#include "gsc_help_window.h"
 #include "gsc_preferences_window.h"
 #include "gsc_executor_log_window.h"
 #include "gsc_executor_error_dialog.h"  // gsc_executor_error_dialog_show
 
 #include "gsc_main_window_iconview.h"
 #include "gsc_main_window.h"
+#include "gsc_add_device_window.h"
+#include "applib/executor_factory.h"
 
 
 
@@ -49,9 +56,29 @@ GscMainWindow::GscMainWindow(BaseObjectType* gtkcobj, const app_ui_res_ref_t& re
 	// iconview, gtkuimanager stuff (menus), custom labels
 	create_widgets();
 
+	// Size
+	{
+		int def_size_w = 0, def_size_h = 0;
+		rconfig::get_data("gui/main_window/default_size_w", def_size_w);
+		rconfig::get_data("gui/main_window/default_size_h", def_size_h);
+		if (def_size_w > 0 && def_size_h > 0) {
+			set_default_size(def_size_w, def_size_h);
+		}
+	}
+
 	// show the window first, scan later
 	show();
 
+	// Position (after the window has been shown)
+	{
+		int pos_x = 0, pos_y = 0;
+		rconfig::get_data("gui/main_window/default_pos_x", pos_x);
+		rconfig::get_data("gui/main_window/default_pos_y", pos_y);
+		if (pos_x > 0 && pos_y > 0) {  // to avoid situations where positions are not supported
+			this->move(pos_x, pos_y);
+		}
+	}
+
 	while (Gtk::Main::events_pending())  // allow the window to show
 		Gtk::Main::iteration();
 
@@ -102,13 +129,25 @@ GscMainWindow::GscMainWindow(BaseObjectType* gtkcobj, const app_ui_res_ref_t& re
 			break;
 		}
 
+		{
+			// We require this version at runtime to support --get=all.
+			const double minimum_req_version = 5.43;
+			double version_double = 0;
+			if (hz::string_is_numeric<double>(version, version_double, false)) {
+				if (version_double < minimum_req_version) {
+					error_msg = "Smartctl version " + version + " found, " + hz::number_to_string(minimum_req_version) + " required.";
+					break;
+				}
+			}
+		}
+
 	} while (false);
 
 	bool smartctl_valid = error_msg.empty();
 	if (!smartctl_valid) {
 		gsc_executor_error_dialog_show("There was an error while executing smartctl",
 				error_msg + "\n\n<i>Please specify the correct smartctl binary in Preferences.</i>",
-				this, show_output_button, true);
+				this, true, show_output_button);
 	}
 
 	// Scan
@@ -118,11 +157,20 @@ GscMainWindow::GscMainWindow(BaseObjectType* gtkcobj, const app_ui_res_ref_t& re
 
 
 
+void GscMainWindow::obj_destroy()
+{
+	// This is needed because for some reason, if any icon is selected,
+	// on_iconview_selection_changed() is called even after the window is deleted,
+	// causing crash on exit.
+	iconview->clear_all();
+}
+
+
 
 void GscMainWindow::populate_iconview(bool smartctl_valid)
 {
 	if (!smartctl_valid) {
-		iconview->empty_view_message = GscMainWindowIconView::message_no_smartctl;
+		iconview->set_empty_view_message(GscMainWindowIconView::message_no_smartctl);
 		iconview->clear_all();  // the message won't be shown without invalidating the region.
 		while (Gtk::Main::events_pending())  // give expose event the time it needs
 			Gtk::Main::iteration();
@@ -132,7 +180,7 @@ void GscMainWindow::populate_iconview(bool smartctl_valid)
 		rescan_devices();  // scan for devices and fill the iconview
 
 	} else {
-		iconview->empty_view_message = GscMainWindowIconView::message_scan_disabled;
+		iconview->set_empty_view_message(GscMainWindowIconView::message_scan_disabled);
 		iconview->clear_all();  // the message won't be shown without invalidating the region.
 		while (Gtk::Main::events_pending())  // give expose event the time it needs
 			Gtk::Main::iteration();
@@ -145,7 +193,14 @@ void GscMainWindow::populate_iconview(bool smartctl_valid)
 		if (rconfig::get_data("/runtime/gui/add_devices_on_startup", load_devices)) {
 			for (unsigned int i = 0; i < load_devices.size(); ++i) {
 				if (!load_devices[i].empty()) {
-					add_device(load_devices[i]);
+					std::vector<std::string> parts;
+					hz::string_split(load_devices[i], "::", parts, false);
+					std::string file = (parts.size() > 0 ? parts.at(0) : "");
+					std::string type_arg = (parts.size() > 1 ? parts.at(1) : "");
+					std::string extra_args = (parts.size() > 2 ? parts.at(2) : "");
+					if (!file.empty()) {
+						add_device(file, type_arg, extra_args);
+					}
 				}
 			}
 		}
@@ -207,14 +262,13 @@ bool GscMainWindow::create_widgets()
 
 	"	<menu action='options_menu'>"
 	"		<menuitem action='" APP_ACTION_NAME(action_executor_log) "' />"
+	"		<menuitem action='" APP_ACTION_NAME(action_update_drivedb) "' />"
 	"		<menuitem action='" APP_ACTION_NAME(action_preferences) "' />"
 	"	</menu>"
 
 	"	<menu action='help_menu'>"
-	"		<menuitem action='" APP_ACTION_NAME(action_general_help) "' />"
-	"		<menuitem action='" APP_ACTION_NAME(action_permission_problems) "' />"
-	"		<menuitem action='" APP_ACTION_NAME(action_how_to_enable_smart) "' />"
-	"		<menuitem action='" APP_ACTION_NAME(action_report_bug) "' />"
+	"		<menuitem action='" APP_ACTION_NAME(action_online_documentation) "' />"
+	"		<menuitem action='" APP_ACTION_NAME(action_support) "' />"
 	"		<menuitem action='" APP_ACTION_NAME(action_about) "' />"
 	"	</menu>"
 
@@ -255,24 +309,20 @@ bool GscMainWindow::create_widgets()
 
 	actiongroup_main->add(Gtk::Action::create("device_menu", "_Device"));
 
-#if APP_GTKMM_CHECK_VERSION(2, 8, 0)  // Gtk::Stock::INFO is since gtkmm 2.8
 		action = Gtk::Action::create(APP_ACTION_NAME(action_view_details), Gtk::Stock::INFO, "_View details",
 				"View detailed information");
-#else
-		action = Gtk::Action::create(APP_ACTION_NAME(action_view_details), "_View details", "View detailed information");
-#endif
 		actiongroup_device->add((action_map[action_view_details] = action), Gtk::AccelKey("<control>V"),
 				sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_view_details));
 
 		action = Gtk::ToggleAction::create(APP_ACTION_NAME(action_enable_smart), "Enable S_MART",
 				"Toggle SMART status. The status will be preserved at least until reboot (unless you toggle it again).");
-		action->connect_proxy(*lookup_widget<Gtk::CheckButton*>("status_smart_enabled_check"));
+		lookup_widget<Gtk::CheckButton*>("status_smart_enabled_check")->set_related_action(action);
 		actiongroup_device->add((action_map[action_enable_smart] = action), Gtk::AccelKey("<control>M"),
 				sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_enable_smart));
 
 		action = Gtk::ToggleAction::create(APP_ACTION_NAME(action_enable_aodc), "Enable Auto O_ffline Data Collection",
 				"Toggle Automatic Offline Data Collection which will update \"offline\" SMART attributes every four hours");
-		action->connect_proxy(*lookup_widget<Gtk::CheckButton*>("status_aodc_enabled_check"));
+		lookup_widget<Gtk::CheckButton*>("status_aodc_enabled_check")->set_related_action(action);
 		actiongroup_device->add((action_map[action_enable_aodc] = action), Gtk::AccelKey("<control>F"),
 				sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_enable_aodc));
 
@@ -318,27 +368,23 @@ bool GscMainWindow::create_widgets()
 		actiongroup_main->add((action_map[action_executor_log] = action),
 				sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_executor_log));
 
+		action = Gtk::Action::create(APP_ACTION_NAME(action_update_drivedb), "Update Drive Database");
+		actiongroup_main->add((action_map[action_update_drivedb] = action),
+				sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_update_drivedb));
+
 		action = Gtk::Action::create(APP_ACTION_NAME(action_preferences), Gtk::Stock::PREFERENCES);
 		actiongroup_main->add((action_map[action_preferences] = action), Gtk::AccelKey("<alt>P"),
 				sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_preferences));
 
 	actiongroup_main->add(Gtk::Action::create("help_menu", "_Help"));
 
-		action = Gtk::Action::create(APP_ACTION_NAME(action_general_help), Gtk::Stock::HELP, "A_ll Help Topics");
-		actiongroup_main->add((action_map[action_general_help] = action), Gtk::AccelKey("F1"),
-				sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_general_help));
+		action = Gtk::Action::create(APP_ACTION_NAME(action_online_documentation), Gtk::Stock::HELP);
+		actiongroup_main->add((action_map[action_online_documentation] = action), Gtk::AccelKey("F1"),
+				sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_online_documentation));
 
-		action = Gtk::Action::create(APP_ACTION_NAME(action_permission_problems), "Resolving _Permission Problems");
-		actiongroup_main->add((action_map[action_permission_problems] = action),
-				sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_permission_problems));
-
-		action = Gtk::Action::create(APP_ACTION_NAME(action_how_to_enable_smart), "How to _Enable SMART Permanently");
-		actiongroup_main->add((action_map[action_how_to_enable_smart] = action),
-				sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_how_to_enable_smart));
-
-		action = Gtk::Action::create(APP_ACTION_NAME(action_report_bug), "_Reporting Bugs");
-		actiongroup_main->add((action_map[action_report_bug] = action),
-				sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_report_bug));
+		action = Gtk::Action::create(APP_ACTION_NAME(action_support), "Support");
+		actiongroup_main->add((action_map[action_support] = action),
+				sigc::bind(sigc::mem_fun(*this, &self_type::on_action_activated), action_support));
 
 		action = Gtk::Action::create(APP_ACTION_NAME(action_about), Gtk::Stock::ABOUT);
 		actiongroup_main->add((action_map[action_about] = action),
@@ -368,14 +414,17 @@ bool GscMainWindow::create_widgets()
 	// add some more accelerators (in addition to existing ones)
 	Gtk::Widget* rescan_item = ui_manager->get_widget("/main_menubar/device_menu/" APP_ACTION_NAME(action_rescan_devices));
 	if (rescan_item)
-		rescan_item->add_accelerator("activate", get_accel_group(), GDK_F5, Gdk::ModifierType(0), Gtk::AccelFlags(0));
+		rescan_item->add_accelerator("activate", get_accel_group(), GDK_KEY_F5, Gdk::ModifierType(0), Gtk::AccelFlags(0));
 
 
 	// look after the created widgets
-	Gtk::VBox* menubar_vbox = lookup_widget<Gtk::VBox*>("menubar_vbox");
+	Gtk::Box* menubar_vbox = lookup_widget<Gtk::Box*>("menubar_vbox");
 	Gtk::Widget* menubar = ui_manager->get_widget("/main_menubar");
-	if (menubar_vbox && menubar)
-		menubar_vbox->pack_start(*menubar, Gtk::PACK_SHRINK);
+	if (menubar_vbox && menubar) {
+		menubar_vbox->pack_start(*menubar, Gtk::PACK_EXPAND_WIDGET);
+		menubar->set_hexpand(true);
+// 		menubar->set_halign(Gtk::ALIGN_START);
+	}
 
 
 	// Set tooltips on menu items - gtk does that only on toolbar items.
@@ -397,25 +446,28 @@ bool GscMainWindow::create_widgets()
 	// ----------------------------------------- Labels
 
 	// create and add labels
-	Gtk::HBox* name_label_box = lookup_widget<Gtk::HBox*>("status_name_label_hbox");
+	Gtk::Box* name_label_box = lookup_widget<Gtk::Box*>("status_name_label_hbox");
 	if (name_label_box) {
-		name_label = Gtk::manage(new WrappingLabel("No drive selected", Gtk::ALIGN_LEFT));
+		name_label = Gtk::manage(new Gtk::Label("No drive selected", Gtk::ALIGN_START));
+		name_label->set_line_wrap(true);
 		name_label->set_selectable(true);
 		name_label->show();
 		name_label_box->pack_start(*name_label, true, true);
 	}
 
-	Gtk::HBox* health_label_box = lookup_widget<Gtk::HBox*>("status_health_label_hbox");
+	Gtk::Box* health_label_box = lookup_widget<Gtk::Box*>("status_health_label_hbox");
 	if (health_label_box) {
-		health_label = Gtk::manage(new WrappingLabel("No drive selected", Gtk::ALIGN_LEFT));
+		health_label = Gtk::manage(new Gtk::Label("No drive selected", Gtk::ALIGN_START));
+		health_label->set_line_wrap(true);
 		health_label->set_selectable(true);
 		health_label->show();
 		health_label_box->pack_start(*health_label, true, true);
 	}
 
-	Gtk::HBox* family_label_box = lookup_widget<Gtk::HBox*>("status_family_label_hbox");
+	Gtk::Box* family_label_box = lookup_widget<Gtk::Box*>("status_family_label_hbox");
 	if (family_label_box) {
-		family_label = Gtk::manage(new WrappingLabel("No drive selected", Gtk::ALIGN_LEFT));
+		family_label = Gtk::manage(new Gtk::Label("No drive selected", Gtk::ALIGN_START));
+		family_label->set_line_wrap(true);
 		family_label->set_selectable(true);
 		family_label->show();
 		family_label_box->pack_start(*family_label, true, true);
@@ -427,6 +479,28 @@ bool GscMainWindow::create_widgets()
 
 
 
+namespace {
+
+	/// Return true if the user agrees to quit
+	inline bool ask_about_quit_on_test(Gtk::Window& parent)
+	{
+		int status = 0;
+		{
+			Gtk::MessageDialog dialog(parent,
+					"\nOne of the drives is performing a test. Do you really want to quit?\n\n"
+					"<small>The test will continue to run in the background, but you won't be"
+					" able to monitor it using GSmartControl.</small>",
+					true, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_YES_NO, true);
+			status = dialog.run();
+		}
+		return (status == Gtk::RESPONSE_YES);
+	}
+	
+}
+
+
+
+
 // NOTE: Do NOT bind the Glib::RefPtr<Gtk::Action> parameter to this function.
 // Doing so causes valgrind errors on window destroy (and Send Report dialogs on win32).
 // Probably a gtkmm bug. Use action maps or Gtk::Action* instead.
@@ -455,12 +529,7 @@ void GscMainWindow::on_action_activated(GscMainWindow::action_t action_type)
 
 	switch (action_type) {
 		case action_quit:
-			// if at least one drive is having a test performed, disallow.
-			if (this->testing_active()) {
-				gui_show_warn_dialog("Please wait until all tests are finished.", this);
-				break;
-			}
-			app_quit();  // ends the main loop
+			quit_requested();
 			break;
 
 		case action_view_details:
@@ -528,47 +597,31 @@ void GscMainWindow::on_action_activated(GscMainWindow::action_t action_type)
 			break;
 		}
 
+		case action_update_drivedb:
+		{
+			run_update_drivedb();
+			break;
+		}
+
 		case action_preferences:
 		{
 			GscPreferencesWindow* win = GscPreferencesWindow::create();  // destroyed on close
-			// win->set_transient_for(*this);  // for "destroy with parent", always-on-top
+			win->set_transient_for(*this);  // for "destroy with parent", always-on-top
+			win->set_main_window(this);
+			win->set_modal(true);
 			win->show();
 			break;
 		}
 
-		case action_general_help:
+		case action_online_documentation:
 		{
-			GscHelpWindow* win = GscHelpWindow::create();  // destroyed on close
-			// win->set_transient_for(*this);  // for "destroy with parent", always-on-top
-			win->set_topic("Begin");  // special mark, top of the text.
-			win->show();
+			hz::launch_url(gobj(), "https://gsmartcontrol.sourceforge.io/documentation.html");
 			break;
 		}
 
-		case action_permission_problems:
+		case action_support:
 		{
-			GscHelpWindow* win = GscHelpWindow::create();  // destroyed on close
-			// win->set_transient_for(*this);  // for "destroy with parent", always-on-top
-			win->set_topic("Permission Problems");
-			win->show();
-			break;
-		}
-
-		case action_how_to_enable_smart:
-		{
-			GscHelpWindow* win = GscHelpWindow::create();  // destroyed on close
-			// win->set_transient_for(*this);  // for "destroy with parent", always-on-top
-			win->set_topic("Enable SMART Permanently");
-			win->show();
-			break;
-		}
-
-		case action_report_bug:
-		{
-			GscHelpWindow* win = GscHelpWindow::create();  // destroyed on close
-			// win->set_transient_for(*this);  // for "destroy with parent", always-on-top
-			win->set_topic("Reporting Bugs");
-			win->show();
+			hz::launch_url(gobj(), "https://gsmartcontrol.sourceforge.io/support.html");
 			break;
 		}
 
@@ -666,7 +719,7 @@ void GscMainWindow::on_action_enable_aodc_toggled(Gtk::ToggleAction* action)
 
 			Gtk::Button enable_button("_Enable", true);
 			enable_button.set_image(*Gtk::manage(new Gtk::Image(Gtk::Stock::YES, Gtk::ICON_SIZE_BUTTON)));
-			enable_button.set_flags(enable_button.get_flags() | Gtk::CAN_DEFAULT);
+			enable_button.set_can_default(true);
 			enable_button.show_all();
 			dialog.add_action_widget(enable_button, Gtk::RESPONSE_YES);
 			enable_button.grab_default();  // make it the default widget
@@ -812,10 +865,11 @@ void GscMainWindow::set_drive_menu_status(StorageDeviceRefPtr drive)
 			}
 			if ((action = actiongroup_device->get_action(APP_ACTION_NAME(action_remove_virtual_device))))
 				action->set_visible(drive && is_virtual);
-			if ((action = actiongroup_device->get_action(APP_ACTION_NAME(action_enable_smart))))
+			if ((action = actiongroup_device->get_action(APP_ACTION_NAME(action_enable_smart)))) {
 				action->set_sensitive(smart_status != StorageDevice::status_unsupported);
+			}
 			if ((action = actiongroup_device->get_action(APP_ACTION_NAME(action_enable_aodc))))
-				action->set_sensitive(smart_status != StorageDevice::status_unsupported);
+				action->set_sensitive(aodc_status != StorageDevice::status_unsupported);
 		}
 
 
@@ -887,16 +941,19 @@ void GscMainWindow::update_status_widgets()
 		return;
 	}
 
-	std::string device = (drive->get_is_virtual() ? ("Virtual: " + drive->get_virtual_filename()) : drive->get_device());
-	std::string size = drive->get_device_size_str();
-	std::string model = (drive->get_model_name().empty() ? std::string("Unknown model") : drive->get_model_name());
-	std::string family = (drive->get_family_name().empty() ? "Unknown" : drive->get_family_name());
-	std::string family_fallback = (drive->get_family_name().empty() ? model : drive->get_family_name());
+	std::string device = Glib::Markup::escape_text(drive->get_is_virtual() ? ("Virtual: " + drive->get_virtual_filename()) : drive->get_device_with_type());
+	std::string size = Glib::Markup::escape_text(drive->get_device_size_str());
+	std::string model = Glib::Markup::escape_text(drive->get_model_name().empty() ? std::string("Unknown model") : drive->get_model_name());
+	std::string family = Glib::Markup::escape_text(drive->get_family_name().empty() ? "Unknown" : drive->get_family_name());
+	std::string family_fallback = Glib::Markup::escape_text(drive->get_family_name().empty() ? model : drive->get_family_name());
+	std::string drive_letters_str = Glib::Markup::escape_text(drive->format_drive_letters(false));
 
-
-	std::string info_str = device + (size.empty() ? "" : (", " + size)) + (model.empty() ? "" : (", " + model));
+	std::string info_str = device
+			+ (drive_letters_str.empty() ? "" : (" (<b>" + drive_letters_str + "</b>)"))
+			+ (size.empty() ? "" : (", " + size))
+			+ (model.empty() ? "" : (", " + model));
 	if (name_label) {
-		name_label->set_text(info_str);
+		name_label->set_markup(info_str);
 		app_gtkmm_set_widget_tooltip(*name_label, info_str, false);  // in case it doesn't fit
 	}
 
@@ -943,17 +1000,26 @@ void GscMainWindow::rescan_devices()
 	// ignore double-scan (may happen because we use gtk loop iterations here).
 	if (this->scanning_)
 		return;
-	this->scanning_ = true;
 
 	// don't manipulate window sensitiveness here - it breaks things
 	// (cursors, gtk errors pop out, etc...)
 
 	// if at least one drive is having a test performed, disallow.
 	if (this->testing_active()) {
-		gui_show_warn_dialog("Please wait until all tests are finished.", this);
-		return;
+		int status = 0;
+		{
+			Gtk::MessageDialog dialog(*this,
+					"\nThis operation may abort any running tests. Do you wish to continue?",
+					true, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_YES_NO, true);
+			status = dialog.run();
+		}
+		if (status != Gtk::RESPONSE_YES) {
+			return;
+		}
 	}
 
+	this->scanning_ = true;
+
 // 	std::string match_str = rconfig::get_data<std::string>("system/device_match_patterns");
 	std::string blacklist_str = rconfig::get_data<std::string>("system/device_blacklist_patterns");
 
@@ -962,7 +1028,7 @@ void GscMainWindow::rescan_devices()
 // 	hz::string_split(match_str, ';', match_patterns, true);
 	hz::string_split(blacklist_str, ';', blacklist_patterns, true);
 
-	iconview->empty_view_message = GscMainWindowIconView::message_scanning;
+	iconview->set_empty_view_message(GscMainWindowIconView::message_scanning);
 
 	iconview->clear_all();  // clear previous icons, invalidate region to update the message.
 	while (Gtk::Main::events_pending())  // give expose event the time it needs
@@ -976,10 +1042,9 @@ void GscMainWindow::rescan_devices()
 	sd.add_blacklist_patterns(blacklist_patterns);
 
 
-	SmartctlExecutorGuiRefPtr ex(new SmartctlExecutorGui());
-	ex->create_running_dialog(this);  // pass this as parent
+	ExecutorFactoryRefPtr ex_factory(new ExecutorFactory(true, this));  // run it with GUI support
 
-	std::string error_msg = sd.detect_and_fetch_basic_data(drives, ex);  // run it with GUI support
+	std::string error_msg = sd.detect_and_fetch_basic_data(drives, ex_factory);
 
 	bool error = false;
 
@@ -1000,13 +1065,11 @@ void GscMainWindow::rescan_devices()
 	if (!error && !error_msg.empty()) {  // generic scan error. smartctl errors are not reported during scan at all.
 		// we don't show output button here
 		gsc_executor_error_dialog_show("An error occurred while scanning the system",
-				error_msg, this, false);
+				error_msg, this, false, false);
 		error = true;
 
 	// add them anyway, in case the error was only on one drive.
 	} else { // if (!error) {
-		std::sort(drives.begin(), drives.end());  // hard drives first
-
 		// add them to iconview
 		for (unsigned int i = 0; i < drives.size(); ++i) {
 			if (rconfig::get_data<bool>("gui/show_smart_capable_only")) {
@@ -1019,15 +1082,44 @@ void GscMainWindow::rescan_devices()
 	}
 
 	// in case there are no drives in the system.
-	if (iconview->num_icons == 0)
-		iconview->empty_view_message = GscMainWindowIconView::message_no_drives_found;
+	if (iconview->get_num_icons() == 0)
+		iconview->set_empty_view_message(GscMainWindowIconView::message_no_drives_found);
 
 	this->scanning_ = false;
 }
 
 
 
-bool GscMainWindow::add_device(const std::string& file)
+void GscMainWindow::run_update_drivedb()
+{
+	std::string smartctl_binary = get_smartctl_binary();
+
+	if (smartctl_binary.empty()) {
+		gui_show_error_dialog("Error Updating Drive Database", "Smartctl binary is not specified in configuration.", this);
+		return;
+	}
+
+	std::string update_binary;
+	hz::FsPath path(smartctl_binary);
+	if (path.is_absolute()) {
+		update_binary = path.get_dirname() + "/";
+	}
+	update_binary += "update-smart-drivedb";
+	update_binary = Glib::shell_quote(update_binary);
+
+#ifndef _WIN32
+	update_binary = "xterm -hold -e " + update_binary;
+#endif
+
+	hz::scoped_ptr<GError> spawn_error(0, g_error_free);
+	if (!g_spawn_command_line_async(update_binary.c_str(), &spawn_error.get_ref())) {
+		gui_show_error_dialog("Error Updating Drive Database", spawn_error->message, this);
+	}
+}
+
+
+
+bool GscMainWindow::add_device(const std::string& file, const std::string& type_arg, const std::string& extra_args)
 {
 #ifndef _WIN32  	// win32 doesn't have device files, so skip the check
 	hz::File f(file);
@@ -1040,16 +1132,17 @@ bool GscMainWindow::add_device(const std::string& file)
 #endif
 
 	StorageDeviceRefPtr d(new StorageDevice(file));
+	d->set_type_argument(type_arg);
+	d->set_extra_arguments(extra_args);
 	d->set_is_manually_added(true);
 
-	SmartctlExecutorGuiRefPtr ex(new SmartctlExecutorGui());
-	ex->create_running_dialog(this);  // pass this as parent
+	ExecutorFactoryRefPtr ex_factory(new ExecutorFactory(true, this));  // pass this as dialog parent
 
 	std::vector<StorageDeviceRefPtr> tmp_drives;
 	tmp_drives.push_back(d);
 
 	StorageDetector sd;
-	std::string error_msg = sd.fetch_basic_data(tmp_drives, ex, true);  // return its first error
+	std::string error_msg = sd.fetch_basic_data(tmp_drives, ex_factory, true);  // return its first error
 	if (!error_msg.empty()) {
 		gsc_executor_error_dialog_show("An error occurred while adding the device", error_msg, this);
 
@@ -1110,6 +1203,10 @@ bool GscMainWindow::testing_active() const
 
 GscInfoWindow* GscMainWindow::show_device_info_window(StorageDeviceRefPtr drive)
 {
+	if (!drive) {
+		return 0;
+	}
+
 	// if a test is being run on it, disallow.
 	if (drive->get_test_is_active()) {
 		gui_show_warn_dialog("Please wait until the test is finished on this drive.", this);
@@ -1135,7 +1232,7 @@ GscInfoWindow* GscMainWindow::show_device_info_window(StorageDeviceRefPtr drive)
 
 		if (status == Gtk::RESPONSE_YES) {
 			SmartctlExecutorGuiRefPtr ex(new SmartctlExecutorGui());
-			ex->create_running_dialog(this, "Running %s on " + drive->get_device() + "...");
+			ex->create_running_dialog(this, "Running %s on " + drive->get_device_with_type() + "...");
 			std::string error_msg = drive->set_smart_enabled(true, ex);  // run it with GUI support
 
 			if (!error_msg.empty()) {
@@ -1146,10 +1243,10 @@ GscInfoWindow* GscMainWindow::show_device_info_window(StorageDeviceRefPtr drive)
 
 
 	// Virtual drives are parsed at load time.
-	// Parse non-virtual drives here.
-	if (!drive->get_is_virtual()) {
+	// Parse non-virtual, smart-supporting drives here.
+	if (!drive->get_is_virtual() && drive->get_smart_status() != StorageDevice::status_unsupported) {
 		SmartctlExecutorGuiRefPtr ex(new SmartctlExecutorGui());
-		ex->create_running_dialog(this, "Running %s on " + drive->get_device() + "...");
+		ex->create_running_dialog(this, "Running %s on " + drive->get_device_with_type() + "...");
 		std::string error_msg = drive->fetch_data_and_parse(ex);  // run it with GUI support
 
 		if (!error_msg.empty()) {
@@ -1162,8 +1259,9 @@ GscInfoWindow* GscMainWindow::show_device_info_window(StorageDeviceRefPtr drive)
 	// If the drive output wasn't fully parsed (happens with e.g. scsi and
 	// usb devices), only very basic info is available and there's no point
 	// in showing this window. - for both virtual and non-virtual.
-	if (!drive->get_fully_parsed()) {
-		gui_show_warn_dialog("No additional information is available for this drive.", this);
+	if (drive->get_parse_status() == StorageDevice::parse_status_none) {
+		gsc_no_info_dialog_show("No additional information is available for this drive.",
+				"", this, false, drive->get_info_output(), "Smartctl Output", drive->get_save_filename());
 		return 0;
 	}
 
@@ -1182,69 +1280,23 @@ GscInfoWindow* GscMainWindow::show_device_info_window(StorageDeviceRefPtr drive)
 
 
 
+void GscMainWindow::show_prefs_updated_message()
+{
+	iconview->set_empty_view_message(GscMainWindowIconView::message_please_rescan);
+	iconview->clear_all();  // the message won't be shown without invalidating the region.
+	while (Gtk::Main::events_pending())  // give expose event the time it needs
+		Gtk::Main::iteration();
+}
+
+
+
 
 void GscMainWindow::show_add_device_chooser()
 {
-#ifdef _WIN32
-
-	// A simple text entry dialog, since there are no device files on win32.
-
-	static std::string last_str;
-	if (last_str.empty())
-		last_str = "pd0";
-
-	std::string dev;
-	if (gui_show_text_entry_dialog("Add Device", "Enter a device name",
-			"For example, pd0 means the first physical drive",
-			dev, last_str, this, false)) {
-		last_str = dev;  // safe for the future
-		this->add_device(dev);  // both the GUI and the API is in utf-8, no conversion is necessary.
-	}
-
-
-#else  // non-win32:
-
-	// Show a file chooser
-
-	static std::string last_dir;
-	if (last_dir.empty())
-		last_dir = "/dev";
-
-	Gtk::FileChooserDialog dialog(*this, "Choose Device File...",
-			Gtk::FILE_CHOOSER_ACTION_OPEN);
-
-	// Add response buttons the the dialog
-	dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
-	dialog.add_button(Gtk::Stock::OPEN, Gtk::RESPONSE_ACCEPT);
-
-	if (!last_dir.empty())
-		dialog.set_current_folder(last_dir);
-
-	// Show the dialog and wait for a user response
-	int result = dialog.run();  // the main cycle blocks here
-
-	// Handle the response
-	switch (result) {
-		case Gtk::RESPONSE_ACCEPT:
-		{
-			last_dir = dialog.get_current_folder();  // safe for the future
-
-			std::string file = dialog.get_filename();  // in fs encoding
-			this->add_device(file);
-			break;
-		}
-
-		case Gtk::RESPONSE_CANCEL: case Gtk::RESPONSE_DELETE_EVENT:
-			// nothing, the dialog is closed already
-			break;
-
-		default:
-			debug_out_error("app", DBG_FUNC_MSG << "Unknown dialog response code: " << result << ".\n");
-			break;
-	}
-
-#endif  // non-win32
-
+	GscAddDeviceWindow* window = GscAddDeviceWindow::create();
+	window->set_main_window(this);
+	window->set_transient_for(*this);
+	window->show();
 }
 
 
@@ -1252,7 +1304,35 @@ void GscMainWindow::show_add_device_chooser()
 void GscMainWindow::show_load_virtual_file_chooser()
 {
 	static std::string last_dir;
+	if (last_dir.empty()) {
+		rconfig::get_data("gui/drive_data_open_save_dir", last_dir);
+	}
+	int result = 0;
 
+	Glib::RefPtr<Gtk::FileFilter> specific_filter = Gtk::FileFilter::create();
+	specific_filter->set_name("Text Files");
+	specific_filter->add_pattern("*.txt");
+
+	Glib::RefPtr<Gtk::FileFilter> all_filter = Gtk::FileFilter::create();
+	all_filter->set_name("All Files");
+	all_filter->add_pattern("*");
+
+#if GTK_CHECK_VERSION(3, 20, 0)
+	hz::scoped_ptr<GtkFileChooserNative> dialog(gtk_file_chooser_native_new(
+			"Load Data From...", this->gobj(), GTK_FILE_CHOOSER_ACTION_OPEN, NULL, NULL), g_object_unref);
+
+	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog.get()), specific_filter->gobj());
+	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog.get()), all_filter->gobj());
+
+	gtk_file_chooser_set_select_multiple(GTK_FILE_CHOOSER(dialog.get()), true);
+
+	if (!last_dir.empty()) {
+		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog.get()), last_dir.c_str());
+	}
+
+	result = gtk_native_dialog_run(GTK_NATIVE_DIALOG(dialog.get()));
+
+#else
 	Gtk::FileChooserDialog dialog(*this, "Load Data From...",
 			Gtk::FILE_CHOOSER_ACTION_OPEN);
 
@@ -1260,20 +1340,43 @@ void GscMainWindow::show_load_virtual_file_chooser()
 	dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
 	dialog.add_button(Gtk::Stock::OPEN, Gtk::RESPONSE_ACCEPT);
 
+	dialog.add_filter(specific_filter);
+	dialog.add_filter(all_filter);
+
+	dialog.set_select_multiple(true);
+
 	if (!last_dir.empty())
 		dialog.set_current_folder(last_dir);
 
 	// Show the dialog and wait for a user response
-	int result = dialog.run();  // the main cycle blocks here
+	result = dialog.run();  // the main cycle blocks here
+#endif
 
 	// Handle the response
 	switch (result) {
 		case Gtk::RESPONSE_ACCEPT:
 		{
-			last_dir = dialog.get_current_folder();  // safe for the future
+			std::vector<std::string> files;
 
-			std::string file = dialog.get_filename();  // in fs encoding
-			this->add_virtual_drive(file);
+#if GTK_CHECK_VERSION(3, 20, 0)
+			hz::scoped_ptr<GSList> file_slist(gtk_file_chooser_get_filenames(GTK_FILE_CHOOSER(dialog.get())), g_slist_free);
+			GSList* iterator = file_slist.get();
+			while(iterator) {
+				files.push_back(app_ustring_from_gchar((gchar*)iterator->data));
+				iterator = g_slist_next(iterator);
+			}
+#else
+			files = dialog.get_filenames();  // in fs encoding
+#endif
+			if (!files.empty()) {
+				last_dir = hz::path_get_dirname(files.front());
+			}
+			rconfig::set_data("gui/drive_data_open_save_dir", last_dir);
+			for (size_t i = 0; i < files.size(); ++i) {
+				if (hz::File(files[i]).is_file()) {  // file chooser returns selected directories as well, ignore them.
+					this->add_virtual_drive(files[i]);
+				}
+			}
 			break;
 		}
 
@@ -1289,20 +1392,44 @@ void GscMainWindow::show_load_virtual_file_chooser()
 
 
 
+bool GscMainWindow::quit_requested()
+{
+	// if at least one drive is having a test performed, disallow.
+	if (this->testing_active()) {
+		if (!ask_about_quit_on_test(*this)) {
+			return true;  // handled
+		}
+	}
+
+	// window size / pos
+	{
+		int window_w = 0, window_h = 0;
+		get_size(window_w, window_h);
+		rconfig::set_data("gui/main_window/default_size_w", window_w);
+		rconfig::set_data("gui/main_window/default_size_h", window_h);
+
+		int pos_x = 0, pos_y = 0;
+		get_position(pos_x, pos_y);
+		rconfig::set_data("gui/main_window/default_pos_x", pos_x);
+		rconfig::set_data("gui/main_window/default_pos_y", pos_y);
+	}
+
+	app_quit();  // ends the main loop
+
+	return true;  // event handled, don't call default virtual handler
+}
+
+
 
 // by default, delete_event calls hide().
 bool GscMainWindow::on_delete_event_before(GdkEventAny* e)
 {
-	// if at least one drive is having a test performed, disallow.
-	if (this->testing_active()) {
-		gui_show_warn_dialog("Please wait until all tests are finished.", this);
-		return true;  // handled
-	}
-	app_quit();  // ends the main loop
-	return true;  // event handled, don't call default virtual handler
+	return quit_requested();
 }
 
 
 
 
 
+
+/// @}
