@@ -1,87 +1,111 @@
 /**************************************************************************
  Copyright:
-      (C) 2008 - 2009  Alexander Shaduri <ashaduri 'at' gmail.com>
+      (C) 2008 - 2012  Alexander Shaduri <ashaduri 'at' gmail.com>
  License: See LICENSE_gsmartcontrol.txt
 ***************************************************************************/
+/// \file
+/// \author Alexander Shaduri
+/// \ingroup gsc
+/// \weakgroup gsc
+/// @{
 
 #ifndef GSC_MAIN_WINDOW_ICONVIEW_H
 #define GSC_MAIN_WINDOW_ICONVIEW_H
 
 #include <gtkmm.h>
-#include <gtkmm/icontheme.h>  // must include manually for older gtkmm (at least 2.6, but not 2.12)
 #include <vector>
 #include <cmath>  // std::floor
-
-#if APP_GTKMM_CHECK_VERSION(2, 8, 0)
-	#include <gtk/gtk.h>  // gtk_icon_view_set_cursor, GTK_ICON_VIEW
-#endif
+#include <cairomm/cairomm.h>
 
 #include "hz/string_algo.h"  // string_join
 #include "hz/debug.h"
 #include "hz/data_file.h"  // data_file_find
+#include "applib/app_gtkmm_utils.h"
+#include "applib/storage_property_colors.h"
 
 #include "gsc_main_window.h"
+#include "rconfig/rconfig_mini.h"
 
 
 
-
-// Note: The IconView must have a fixed icon width set (e.g. in glade file).
-// Otherwise, it doesn't re-compute it when clearing and adding new icons.
-
+/// The icon view of the main window (shows a drive list).
+/// Note: The IconView must have a fixed icon width set (e.g. in .ui file).
+/// Otherwise, it doesn't re-compute it when clearing and adding new icons.
 class GscMainWindowIconView : public Gtk::IconView {
 	public:
 
-		typedef GscMainWindowIconView self_type;  // needed for CONNECT_VIRTUAL
+		typedef GscMainWindowIconView self_type;  ///< Self type, needed for CONNECT_VIRTUAL
 
+		/// Message type to show
 		enum message_t {
-			message_none,
-			message_scan_disabled,
-			message_scanning,
-			message_no_drives_found,
-			message_no_smartctl
+			message_none,  ///< No message
+			message_scan_disabled,  ///< Scanning is disabled
+			message_scanning,  ///< Scanning drives...
+			message_no_drives_found,  ///< No drives found
+			message_no_smartctl,  ///< No smartctl installed
+			message_please_rescan,  ///< Re-scan to see the drives
 		};
 
 
-		// glade/gtkbuilder needs this constructor
+		/// Constructor, GtkBuilder needs this.
 		GscMainWindowIconView(BaseObjectType* gtkcobj, const app_ui_res_ref_t& ref_ui)
 				: Gtk::IconView(gtkcobj), num_icons(0), main_window(0), empty_view_message(message_none)
 		{
-			APP_GTKMM_CONNECT_VIRTUAL(expose_event);  // make sure the event handler is called
-
 			columns.add(col_name);  // we can use the col_name variable by value after this.
 			this->set_markup_column(col_name);
 
 			columns.add(col_description);
 
 			columns.add(col_pixbuf);
+
+#if GTK_CHECK_VERSION(3, 10, 0)
+			// For high quality rendering with GDK_SCALE=2
+			this->pack_start(cell_renderer_pixbuf, false);
+			this->set_cell_data_func(cell_renderer_pixbuf,
+					sigc::mem_fun(this, &GscMainWindowIconView::on_cell_data_render));
+#else
 			this->set_pixbuf_column(col_pixbuf);
+#endif
 
 			columns.add(col_drive_ptr);
 
+			columns.add(col_populated);
+
 			// create a Tree Model
 			ref_list_model = Gtk::ListStore::create(columns);
 // 			ref_list_model->set_sort_column(col_name, Gtk::SORT_ASCENDING);
 			this->set_model(ref_list_model);
 
 			// we add it here because list model must be created already
-			gtkmm_set_iconview_tooltip_column(this, col_description, ref_list_model);
+			set_tooltip_column(col_description.index());
 
 
 			// icons
-			Glib::RefPtr<Gtk::IconTheme> default_icon_theme = Gtk::IconTheme::get_default();
+			Glib::RefPtr<Gtk::IconTheme> default_icon_theme;
+			try {
+				default_icon_theme = Gtk::IconTheme::get_default();
+			} catch (...) {
+				// nothing
+			}
 
-			// Try Gnome icons icons first, they are usually more consistent with Gnome desktop
-			// (this should work only if gnome-settings-daemon is running, so no harm on other desktops).
-			// The problem with stock gtk icons is that they are not available in 48x48 size, so
-			// confirm all sizes manually.
+			// Adwaita's drive-harddisk icons are really small at 48, so 64 is better.
+			// Plus, it scales well to 128 and 256 (if using GDK_SCALE).
+			const int icon_size = 64;
 
-			if (app_gtkmm_icon_theme_has_icon(default_icon_theme, "gnome-dev-harddisk", 48))
-				hd_icon = default_icon_theme->load_icon("gnome-dev-harddisk", 48, Gtk::IconLookupFlags(0));
-
-			if (!hd_icon && app_gtkmm_icon_theme_has_icon(default_icon_theme, "gtk-harddisk", 48))
-				hd_icon = default_icon_theme->load_icon("gtk-harddisk", 48, Gtk::IconLookupFlags(0));
-
-			if (!hd_icon) {  // still no luck, use bundled ones.
+			// Try XDG version first
+#if APP_GTKMM_CHECK_VERSION(3, 10, 0)
+			try {
+				hd_icon = default_icon_theme->load_icon("drive-harddisk", icon_size, get_scale_factor(), Gtk::IconLookupFlags(0));
+			} catch (...) { }  // ignore exceptions
+#endif
+			if (!hd_icon) {
+				// Before gtk 3.10 it was called gtk-harddisk.
+				try {
+					hd_icon = default_icon_theme->load_icon("gtk-harddisk", icon_size, Gtk::IconLookupFlags(0));
+				} catch (...) { }  // ignore exceptions
+			}
+			if (!hd_icon) {
+				// Still no luck, use bundled ones.
 				std::string icon_file = hz::data_file_find("icon_hdd.png");
 				if (!icon_file.empty()) {
 					try {
@@ -90,18 +114,20 @@ class GscMainWindowIconView : public Gtk::IconView {
 				}
 			}
 
-			// last resort. gtk has gtk-harddisk built-in. it may not be the right size, but what else can we do?
-			if (!hd_icon)
-				hd_icon = default_icon_theme->load_icon("gtk-harddisk", 48, Gtk::IconLookupFlags(0));
-
-
-			if (app_gtkmm_icon_theme_has_icon(default_icon_theme, "gnome-dev-cdrom", 48))
-				cddvd_icon = default_icon_theme->load_icon("gnome-dev-cdrom", 48, Gtk::IconLookupFlags(0));
-
-			if (!cddvd_icon && app_gtkmm_icon_theme_has_icon(default_icon_theme, "gtk-cdrom", 48))
-				cddvd_icon = default_icon_theme->load_icon("gtk-cdrom", 48, Gtk::IconLookupFlags(0));
-
-			if (!cddvd_icon) {  // still no luck, use bundled ones.
+			// Try XDG version first
+#if APP_GTKMM_CHECK_VERSION(3, 10, 0)
+			try {
+				cddvd_icon = default_icon_theme->load_icon("media-optical", icon_size, get_scale_factor(), Gtk::IconLookupFlags(0));
+			} catch (...) { }  // ignore exceptions
+#endif
+			if (!cddvd_icon) {
+				// Before gtk 3.10 it was called gtk-cdrom.
+				try {
+					cddvd_icon = default_icon_theme->load_icon("gtk-cdrom", icon_size, Gtk::IconLookupFlags(0));
+				} catch (...) { }  // ignore exceptions
+			}
+			if (!cddvd_icon) {
+				// Still no luck, use bundled ones.
 				std::string icon_file = hz::data_file_find("icon_cddvd.png");
 				if (!icon_file.empty()) {
 					try {
@@ -110,10 +136,6 @@ class GscMainWindowIconView : public Gtk::IconView {
 				}
 			}
 
-			if (!cddvd_icon)
-				cddvd_icon = default_icon_theme->load_icon("gtk-cdrom", 48, Gtk::IconLookupFlags(0));
-
-
 			this->signal_item_activated().connect(sigc::mem_fun(*this,
 					&self_type::on_iconview_item_activated) );
 
@@ -125,34 +147,42 @@ class GscMainWindowIconView : public Gtk::IconView {
 		}
 
 
-
+		/// Set the parent window
 		void set_main_window(GscMainWindow* w)
 		{
 			main_window = w;
 		}
 
 
-
-		// override default event handler
-		bool on_expose_event_before(GdkEventExpose* exp_event)
+		/// Set the message type to display when there are no icons to show
+		void set_empty_view_message(message_t message)
 		{
-			// this->on_expose_event(exp_event);  // parent's stuff
+			empty_view_message = message;
+		}
 
+
+		/// Get the number of icons currently displayed
+		unsigned int get_num_icons() const
+		{
+			return num_icons;
+		}
+
+
+		// Overridden from Gtk::Widget
+		bool on_draw(const Cairo::RefPtr<Cairo::Context>& cr)
+		{
 			if (empty_view_message != message_none && this->num_icons == 0) {  // no icons
-				Glib::RefPtr<Gdk::Drawable> win = Glib::wrap(exp_event->window, true);
-	// 			Cairo::RefPtr<Cairo::Context> cr = win->create_cairo_context();
-				Glib::RefPtr<Gdk::GC> gc = Gdk::GC::create(win);
-
 				std::string msg;
 				switch(empty_view_message) {
 					case message_scan_disabled: msg = "Automatic scanning is disabled.\nPress Ctrl+R to scan manually."; break;
 					case message_scanning: msg = "Scanning system, please wait..."; break;
 					case message_no_drives_found: msg = "No drives found."; break;
 					case message_no_smartctl: msg = "Please specify the correct smartctl binary in\nPreferences and press Ctrl-R to re-scan."; break;
+					case message_please_rescan: msg = "Preferences changed.\nPress Ctrl-R to re-scan."; break;
 					default: msg = "[error - invalid message]";
 				}
 
-				Glib::RefPtr<Pango::Layout> layout = create_pango_layout("");
+				Glib::RefPtr<Pango::Layout> layout = this->create_pango_layout("");
 				layout->set_alignment(Pango::ALIGN_CENTER);
 				layout->set_markup(msg);
 
@@ -161,15 +191,55 @@ class GscMainWindowIconView : public Gtk::IconView {
 
 				int pos_x = (get_allocation().get_width() - layout_w) / 2;
 				int pos_y = (get_allocation().get_height() - layout_h) / 2;
+				cr->move_to(pos_x, pos_y);
 
-				win->draw_layout(gc, pos_x, pos_y, layout);
+				layout->show_in_cairo_context(cr);
+
+				return true;
 			}
 
-			return false;  // without false the icons don't get drawn...
+			return Gtk::IconView::on_draw(cr);
 		}
 
 
 
+#if GTK_CHECK_VERSION(3, 10, 0)
+		/// Cell data renderer (needed for high quality icons in GDK_SCALE=2).
+		/// We have to use Cairo surfaces, because pixbufs are scaled by GtkIconView.
+		void on_cell_data_render(const Gtk::TreeModel::const_iterator& iter)
+		{
+			Gtk::TreeRow row = *iter;
+			if (!row[col_populated]) {  // protect against rendering incomplete model entry
+				return;
+			}
+
+			Glib::RefPtr<Gdk::Pixbuf> pixbuf = row[col_pixbuf];
+			if (!pixbuf) {
+				return;
+			}
+
+			// Gtkmm property_surface() doesn't work, so use plain C.
+			// https://bugzilla.gnome.org/show_bug.cgi?id=788513
+			// Also, Gtkmm doesn't have gdk_cairo_surface_create_from_pixbuf() wrapper.
+			// https://bugzilla.gnome.org/show_bug.cgi?id=788533
+
+// 			Cairo::Format format = Cairo::FORMAT_ARGB32;
+// 			if (pixbuf->get_n_channels() == 3) {
+// 				format = Cairo::FORMAT_RGB24;
+// 			}
+// 			Cairo::RefPtr<Cairo::Surface> surface = get_window()->create_similar_image_surface(
+// 					format, pixbuf->get_width(), pixbuf->get_height(), get_scale_factor());
+// 			cell_renderer_pixbuf.property_surface().set_value(surface);
+
+			// gdk_cairo_surface_create_from_pixbuf() (and create_similar_image_surface()) from gtk 3.10.
+			cairo_surface_t* surface = gdk_cairo_surface_create_from_pixbuf(pixbuf->gobj(), get_scale_factor(), get_window()->gobj());
+			g_object_set(G_OBJECT(cell_renderer_pixbuf.gobj()), "surface", surface, NULL);
+			cairo_surface_destroy(surface);
+		}
+#endif
+
+
+		/// Add a drive entry to the icon view
 		void add_entry(StorageDeviceRefPtr drive, bool scroll_to_it = false)
 		{
 			if (!drive)
@@ -180,20 +250,20 @@ class GscMainWindowIconView : public Gtk::IconView {
 
 			this->decorate_entry(row);
 
+			row[col_populated] = true;  // triggers rendering
+
 			drive->signal_changed.connect(sigc::mem_fun(this, &GscMainWindowIconView::on_drive_changed));
 
 			if (scroll_to_it) {
 				Gtk::TreeModel::Path tpath(row);
 				// scroll_to_path() and set/get_cursor() are since gtkmm 2.8.
-#if APP_GTKMM_CHECK_VERSION(2, 8, 0)
+
 				this->scroll_to_path(tpath, true, 0.5, 0.5);
 				// select it (keyboard & selection)
 				Gtk::CellRenderer* cell = 0;
 				if (this->get_cursor(cell) && cell) {
-					// gtkmm's set_cursor() is undefined (but declared) in 2.8, so use gtk variant.
-					gtk_icon_view_set_cursor(GTK_ICON_VIEW(this->gobj()), tpath.gobj(), cell->gobj(), false);
+					this->set_cursor(tpath, *cell, false);
 				}
-#endif
 				this->select_path(tpath);  // highlight it
 			}
 
@@ -202,7 +272,8 @@ class GscMainWindowIconView : public Gtk::IconView {
 
 
 
-		// this should be called to update the icon of already refreshed drive
+		/// Decorate a drive entry (colorize it if it has errors, etc...).
+		/// This should be called to update the icon of already refreshed drive.
 		void decorate_entry(Gtk::TreePath model_path)
 		{
 			if (model_path.empty())
@@ -214,25 +285,67 @@ class GscMainWindowIconView : public Gtk::IconView {
 
 
 
-		// this should be called to update the icon of already refreshed drive
+		/// Decorate a drive entry (colorize it if it has errors, etc...).
+		/// This should be called to update the icon of already refreshed drive.
 		void decorate_entry(Gtk::TreeModel::Row& row)
 		{
 			StorageDeviceRefPtr drive = row[col_drive_ptr];
+			if (!drive) {
+				return;
+			}
 
 			// it needs this space to be symmetric (why?);
-			std::string name;  // = "<big>" + drive->get_device_pretty() + " </big>\n";
+			std::string name;  // = "<big>" + drive->get_device_with_type() + " </big>\n";
+			Glib::ustring drive_letters = Glib::Markup::escape_text(drive->format_drive_letters(false));
+			if (drive_letters.empty()) {
+				drive_letters = "not mounted";
+			}
+			Glib::ustring drive_letters_with_volname = Glib::Markup::escape_text(drive->format_drive_letters(true));
+			if (drive_letters_with_volname.empty()) {
+				drive_letters_with_volname = "not mounted";
+			}
 
 			// note: if this wraps, it becomes left-aligned in gtk <= 2.10.
-			name += (drive->get_model_name().empty() ? std::string("Unknown model") : drive->get_model_name());
-
+			name += (drive->get_model_name().empty() ? Glib::ustring("Unknown model") : Glib::Markup::escape_text(drive->get_model_name()));
+			if (rconfig::get_data<bool>("gui/icons_show_device_name")) {
+				if (!drive->get_is_virtual()) {
+					name += "\n" + Glib::Markup::escape_text(drive->get_device_with_type());
+				#ifdef _WIN32
+					name += " (" + drive_letters + ")";
+				#endif
+				} else if (!drive->get_virtual_filename().empty()) {
+					name += "\n" + Glib::Markup::escape_text(drive->get_virtual_filename());
+				}
+			}
+			if (rconfig::get_data<bool>("gui/icons_show_serial_number") && !drive->get_serial_number().empty()) {
+				name += "\n" + Glib::Markup::escape_text(drive->get_serial_number());
+			}
+			StorageProperty scan_time_prop;
+			if (drive->get_is_virtual()) {
+				scan_time_prop = drive->lookup_property("scan_time");
+				if (!scan_time_prop.value_string.empty()) {
+					name += "\n" + Glib::Markup::escape_text(scan_time_prop.value_string);
+				}
+			}
 
 			std::vector<std::string> tooltip_strs;
 
 			if (drive->get_is_virtual()) {
 				std::string vfile = drive->get_virtual_filename();
 				tooltip_strs.push_back("Loaded from: " + (vfile.empty() ? "[empty]" : Glib::Markup::escape_text(vfile)));
+				if (!scan_time_prop.value_string.empty()) {
+					tooltip_strs.push_back("Scanned on: " + Glib::Markup::escape_text(scan_time_prop.value_string));
+				}
 			} else {
-				tooltip_strs.push_back("Device: <b>" + Glib::Markup::escape_text(drive->get_device()) + "</b>");
+				tooltip_strs.push_back("Device: <b>" + Glib::Markup::escape_text(drive->get_device_with_type()) + "</b>");
+			}
+
+		#ifdef _WIN32
+			tooltip_strs.push_back("Drive letters: <b>" + drive_letters_with_volname + "</b>");
+		#endif
+
+			if (!drive->get_serial_number().empty()) {
+				tooltip_strs.push_back("Serial number: <b>" + Glib::Markup::escape_text(drive->get_serial_number()) + "</b>");
 			}
 			tooltip_strs.push_back("SMART status: <b>"
 					+ StorageDevice::get_status_name(drive->get_smart_status(), false) + "</b>");
@@ -243,30 +356,37 @@ class GscMainWindowIconView : public Gtk::IconView {
 
 
 			Glib::RefPtr<Gdk::Pixbuf> icon;
-			if (drive->get_type() == StorageDevice::type_cddvd) {
-				icon = cddvd_icon;
-			} else {
-				icon = hd_icon;
+			switch(drive->get_detected_type()) {
+				case StorageDevice::detected_type_cddvd:
+					icon = cddvd_icon;
+					break;
+				case StorageDevice::detected_type_unknown:  // standard HD icon
+				case StorageDevice::detected_type_invalid:
+				case StorageDevice::detected_type_raid:  // TODO a separate icon for this
+					icon = hd_icon;
+					break;
 			}
 
 			StorageProperty health_prop = drive->get_health_property();
 			if (health_prop.warning != StorageProperty::warning_none && health_prop.generic_name == "overall_health") {
-				icon = icon->copy();  // work on a copy
-				if (icon->get_colorspace() == Gdk::COLORSPACE_RGB && icon->get_bits_per_sample() == 8) {
-					int n_channels = icon->get_n_channels();
-					int icon_width = icon->get_width();
-					int icon_height = icon->get_height();
-					int rowstride = icon->get_rowstride();
-					guint8* pixels = icon->get_pixels();
+				if (icon) {
+					icon = icon->copy();  // work on a copy
+					if (icon->get_colorspace() == Gdk::COLORSPACE_RGB && icon->get_bits_per_sample() == 8) {
+						int n_channels = icon->get_n_channels();
+						int icon_width = icon->get_width();
+						int icon_height = icon->get_height();
+						int rowstride = icon->get_rowstride();
+						guint8* pixels = icon->get_pixels();
 
-					guint8* p = 0;
-					for (int y = 0; y < icon_height; ++y) {
-						for (int x = 0; x < icon_width; ++x) {
-							p = pixels + y * rowstride + x * n_channels;
-							uint8_t avg = static_cast<uint8_t>(std::floor((p[0] * 0.30) + (p[1] * 0.59) + (p[2] * 0.11) + 0.001 + 0.5));
-							p[0] = avg;  // R
-							p[1] = 0;  // G
-							p[2] = 0;  // B
+						guint8* p = 0;
+						for (int y = 0; y < icon_height; ++y) {
+							for (int x = 0; x < icon_width; ++x) {
+								p = pixels + y * rowstride + x * n_channels;
+								uint8_t avg = static_cast<uint8_t>(std::floor((p[0] * 0.30) + (p[1] * 0.59) + (p[2] * 0.11) + 0.001 + 0.5));
+								p[0] = avg;  // R
+								p[1] = 0;  // G
+								p[2] = 0;  // B
+							}
 						}
 					}
 				}
@@ -291,6 +411,7 @@ class GscMainWindowIconView : public Gtk::IconView {
 
 
 
+		/// Remove drive entry
 		void remove_entry(const Gtk::TreePath& model_path)
 		{
 			Gtk::TreeModel::Row row = *(ref_list_model->get_iter(model_path));
@@ -298,6 +419,8 @@ class GscMainWindowIconView : public Gtk::IconView {
 		}
 
 
+
+		/// Remove selected drive entry
 		void remove_selected_drive()
 		{
 			if (this->get_selected_items().size()) {
@@ -308,13 +431,14 @@ class GscMainWindowIconView : public Gtk::IconView {
 
 
 
+		/// Remove all entries
 		void clear_all()
 		{
 			num_icons = 0;
 			ref_list_model->clear();
 
 			// this is needed to update the label from "disabled" to "scanning"
-			if (this->is_realized()) {
+			if (this->get_realized()) {
 				Gdk::Rectangle rect = this->get_allocation();
 				Glib::RefPtr<Gdk::Window> win = this->get_window();
 				win->invalidate_rect(rect, true);  // force expose event
@@ -323,6 +447,8 @@ class GscMainWindowIconView : public Gtk::IconView {
 		}
 
 
+
+		/// Get selected drive
 		StorageDeviceRefPtr get_selected_drive()
 		{
 			StorageDeviceRefPtr drive = 0;
@@ -335,6 +461,8 @@ class GscMainWindowIconView : public Gtk::IconView {
 		}
 
 
+
+		/// Get tree path by a drive
 		Gtk::TreePath get_path_by_drive(StorageDevice* drive)
 		{
 			Gtk::TreeNodeChildren children = ref_list_model->children();
@@ -349,6 +477,7 @@ class GscMainWindowIconView : public Gtk::IconView {
 
 
 
+		/// Update menu actions in the Drives menu
 		void update_menu_actions()
 		{
 			// if there's nothing selected, disable items from "Drives" menu
@@ -358,14 +487,17 @@ class GscMainWindowIconView : public Gtk::IconView {
 			} else {  // enable drives menu, set proper smart toggles
 				Gtk::TreePath model_path = *(this->get_selected_items().begin());
 				Gtk::TreeModel::Row row = *(ref_list_model->get_iter(model_path));
-				StorageDeviceRefPtr drive = row[col_drive_ptr];
+				if (!row[col_populated]) {  // protect against using incomplete model entry
+					return;
+				}
 
+				StorageDeviceRefPtr drive = row[col_drive_ptr];
 				main_window->set_drive_menu_status(drive);
 			}
 		}
 
 
-		// callback
+		/// Callback
 		void on_iconview_item_activated(const Gtk::TreePath& model_path)
 		{
 			debug_out_info("app", DBG_FUNC << "\n");
@@ -373,13 +505,16 @@ class GscMainWindowIconView : public Gtk::IconView {
 				return;
 
 			Gtk::TreeModel::Row row = *(ref_list_model->get_iter(model_path));
-			StorageDeviceRefPtr drive = row[col_drive_ptr];
+			if (!row[col_populated]) {  // protect against using incomplete model entry
+				return;
+			}
 
+			StorageDeviceRefPtr drive = row[col_drive_ptr];
 			main_window->show_device_info_window(drive);
 		}
 
 
-		// callback
+		/// Callback
 		void on_iconview_selection_changed()
 		{
 			// Must do it here - if done during menu activation, the actions won't work
@@ -390,7 +525,7 @@ class GscMainWindowIconView : public Gtk::IconView {
 		}
 
 
-		// callback
+		/// Callback
 		bool on_iconview_button_press_event(GdkEventButton* event_button)
 		{
 			// select and show popup menu on right-click
@@ -405,14 +540,11 @@ class GscMainWindowIconView : public Gtk::IconView {
 				if (tpath.gobj() && !tpath.empty()) {  // without gobj() check gtkmm 2.6 (but not 2.12) prints lots of errors
 					// move keyboard focus to the icon (just as left-click does)
 
-					// set/get_cursor() are since gtkmm 2.8.
-#if APP_GTKMM_CHECK_VERSION(2, 8, 0)
 					Gtk::CellRenderer* cell = 0;
 					if (this->get_cursor(cell) && cell) {
 						// gtkmm's set_cursor() is undefined (but declared) in 2.8, so use gtk variant.
 						gtk_icon_view_set_cursor(GTK_ICON_VIEW(this->gobj()), tpath.gobj(), cell->gobj(), false);
 					}
-#endif
 
 					// select the icon
 					this->select_path(tpath);
@@ -424,9 +556,11 @@ class GscMainWindowIconView : public Gtk::IconView {
 					this->unselect_all();  // unselect on empty area right-click
 				}
 
-				Gtk::Menu* menu = main_window->get_popup_menu(drive);
-				if (menu)
-					menu->popup(event_button->button, event_button->time);
+				if (drive) {
+					Gtk::Menu* menu = main_window->get_popup_menu(drive);
+					if (menu)
+						menu->popup(event_button->button, event_button->time);
+				}
 
 				return true;  // stop handling
 			}
@@ -437,7 +571,7 @@ class GscMainWindowIconView : public Gtk::IconView {
 		}
 
 
-		// callback attached to StorageDevice
+		/// Callback attached to StorageDevice, updates its view.
 		void on_drive_changed(StorageDevice* drive)
 		{
 			Gtk::TreePath model_path = this->get_path_by_drive(drive);
@@ -447,24 +581,27 @@ class GscMainWindowIconView : public Gtk::IconView {
 		}
 
 
+	private:
 
-		Gtk::TreeModel::ColumnRecord columns;
+		Gtk::TreeModel::ColumnRecord columns;  ///< Model columns
+		Gtk::CellRendererPixbuf cell_renderer_pixbuf;  ///< Cell renderer for icons.
 
-		Gtk::TreeModelColumn<std::string> col_name;
-		Gtk::TreeModelColumn<Glib::ustring> col_description;
-		Gtk::TreeModelColumn<Glib::RefPtr<Gdk::Pixbuf> > col_pixbuf;
-		Gtk::TreeModelColumn<StorageDeviceRefPtr> col_drive_ptr;
+		Gtk::TreeModelColumn<std::string> col_name;  ///< Model column
+		Gtk::TreeModelColumn<Glib::ustring> col_description;  ///< Model column
+		Gtk::TreeModelColumn<Glib::RefPtr<Gdk::Pixbuf> > col_pixbuf;  ///< Model column
+		Gtk::TreeModelColumn<StorageDeviceRefPtr> col_drive_ptr;  ///< Model column
+		Gtk::TreeModelColumn<bool> col_populated;  ///< Model column, indicates whether the model entry has been fully populated.
 
-		Glib::RefPtr<Gtk::ListStore> ref_list_model;
-		unsigned int num_icons;  // track the number of icons, because liststore makes it difficult to count them.
+		Glib::RefPtr<Gtk::ListStore> ref_list_model;  ///< The icon view model
+		unsigned int num_icons;  ///< Track the number of icons, because liststore makes it difficult to count them.
 
 		// available icons
-		Glib::RefPtr<Gdk::Pixbuf> hd_icon;
-		Glib::RefPtr<Gdk::Pixbuf> cddvd_icon;
+		Glib::RefPtr<Gdk::Pixbuf> hd_icon;  ///< Icon pixbuf
+		Glib::RefPtr<Gdk::Pixbuf> cddvd_icon;  ///< Icon pixbuf
 
-		GscMainWindow* main_window;
+		GscMainWindow* main_window;  ///< The main window, our parent
 
-		message_t empty_view_message;  // message displayed when no icons
+		message_t empty_view_message;  ///< Message type to display when not showing any icons
 
 };
 
@@ -476,3 +613,5 @@ class GscMainWindowIconView : public Gtk::IconView {
 
 
 #endif
+
+/// @}
