@@ -2,9 +2,15 @@
  Copyright:
       (C) 1992, 1993  The Regents of the University of California
  License: See LICENSE_bsd-ucb.txt file
-      (C) 2008 - 2009  Alexander Shaduri <ashaduri 'at' gmail.com>
+      (C) 2008 - 2012  Alexander Shaduri <ashaduri 'at' gmail.com>
  License: See LICENSE_zlib.txt file
 ***************************************************************************/
+/// \file
+/// \author The Regents of the University of California
+/// \author Alexander Shaduri
+/// \ingroup hz
+/// \weakgroup hz
+/// @{
 
 #ifndef HZ_ASCII_H
 #define HZ_ASCII_H
@@ -12,8 +18,9 @@
 #include "hz_config.h"  // feature macros
 
 
-// Part of this file (specifically, ascii_strtoi() implementation)
-// is derived from FreeBSD's strtol() and friends.
+/// \file
+/// Part of this file (specifically, ascii_strtoi() implementation)
+/// is derived from FreeBSD's strtol() and friends.
 
 // The original FreeBSD copyright header follows:
 /*-
@@ -69,7 +76,14 @@
 
 #include "type_properties.h"  // type_is_*, type_make_unsigned
 
-
+// Disable silly VS warnings
+#ifdef _MSC_VER
+	#pragma warning(push)
+	#pragma warning(disable: 4800)  // 'int' : forcing value to bool 'true' or 'false' (performance warning)
+	#pragma warning(disable: 4804)  // unsafe use of type 'bool' in operation
+	#pragma warning(disable: 4146)  // unary minus operator applied to unsigned type, result still unsigned
+	#pragma warning(disable: 6328)  // 'const char' passed as parameter '1' when 'unsigned char' is required in call to 'isspace'
+#endif
 
 // Locale-independent functions for ascii manipulation.
 
@@ -79,9 +93,9 @@ namespace hz {
 
 
 
-// Implementation of std::isspace().
-// This function always behaves like the standard function
-// does in Classic locale.
+/// Implementation of std::isspace().
+/// This function always behaves like the standard function
+/// does in Classic locale (regardless of current locale).
 inline bool ascii_isspace(char c)
 {
 	return (c == ' ' || c == '\f' || c == '\n' || c == '\r' || c == '\t' || c == '\v');
@@ -89,11 +103,11 @@ inline bool ascii_isspace(char c)
 
 
 
-// Implementation of std::strtoX() for every native integral type:
-// char (numerical value), wchar_t (numerical value),
-// int, short, long, long long, plus their signed/unsigned variants.
-// This function always behaves like the standard functions do
-// in Classic locale.
+/// Implementation of std::strtoX() functions for every native integral type:
+/// char (numerical value), wchar_t (numerical value),
+/// int, short, long, long long, plus their signed/unsigned variants.
+/// This function always behaves like the standard functions do
+/// in Classic locale.
 template<typename T> inline
 T ascii_strtoi(const char* nptr, char** endptr, int base)
 {
@@ -170,9 +184,17 @@ T ascii_strtoi(const char* nptr, char** endptr, int base)
 	// overflow.
 
 	if (std::numeric_limits<T>::is_signed) {
+#ifdef __GNUC__
+	#pragma GCC diagnostic push
+	#pragma GCC diagnostic ignored "-Wint-in-bool-context"
+	#pragma GCC diagnostic ignored "-Woverflow"
+#endif
 		cutoff = static_cast<unsigned_type>( neg ?
 				( static_cast<unsigned_type>(-(std::numeric_limits<T>::min() + std::numeric_limits<T>::max()))
 				+ std::numeric_limits<T>::max() ) : std::numeric_limits<T>::max() );
+#ifdef __GNUC__
+	#pragma GCC diagnostic pop
+#endif
 		cutlim = static_cast<signed_type>(cutoff % base);
 		cutoff = static_cast<unsigned_type>(cutoff / base);
 	} else {
@@ -232,11 +254,12 @@ namespace internal {
 
 	// proxy functions for system strtod family.
 
-	// we use struct because the errors are easier on the user this way.
+	/// We use struct because the errors are easier on the user this way.
 	template<typename T>
 	struct ascii_strtof_impl { };
 
 
+	/// Specialization
 	template<>
 	struct ascii_strtof_impl<double> {
 		static double func(const char* nptr, char** endptr)
@@ -248,6 +271,7 @@ namespace internal {
 
 	// Note: strtold() and strtof() are C99 / _XOPEN_SOURCE >= 600.
 
+	/// Specialization
 	template<>
 	struct ascii_strtof_impl<float> {
 		static float func(const char* nptr, char** endptr)
@@ -276,7 +300,7 @@ namespace internal {
 			// I'm not sure if a flag should be raised at all.
 			// If anyone knows how to do it portably and reliably, please tell me.
 
-			return val;
+			return float(val);
 #endif
 
 
@@ -284,6 +308,7 @@ namespace internal {
 	};
 
 
+	/// Specialization
 	template<>
 	struct ascii_strtof_impl<long double> {
 		static long double func(const char* nptr, char** endptr)
@@ -328,9 +353,9 @@ namespace internal {
 // on overflow, +/-HUGE_VAL[L|F] is returned, and errno is set to ERANGE.
 // on underflow, 0 is returned and errno is set to ERANGE.
 
-// A unified strtod/strtof/strtold.
-// This function always behaves like the standard functions do
-// in Classic locale.
+/// Implementation of strtod/strtof/strtold for any floating-point type.
+/// This function always behaves like the standard functions do
+/// in Classic locale.
 template<typename T> inline
 T ascii_strtof(const char* nptr, char** endptr)
 {
@@ -363,7 +388,7 @@ T ascii_strtof(const char* nptr, char** endptr)
 	ASSERT(locdata);
 	const char* radix = locdata->decimal_point;
 	ASSERT(radix);
-	std::size_t radix_len = std::strlen(radix);
+	std::size_t radix_len = (radix ? std::strlen(radix) : 0U);
 	ASSERT(radix_len);
 
 	// Check if the locale resembles Classic.
@@ -472,9 +497,11 @@ T ascii_strtof(const char* nptr, char** endptr)
 
 namespace {
 
+	/// Helper struct to choose the correct implementation
 	template<bool IsInt, bool IsFloat>
 	struct ascii_strton_impl { };
 
+	/// Specialization
 	template<bool IsInt>
 	struct ascii_strton_impl<IsInt, true> {
 		template<typename T> inline
@@ -484,6 +511,7 @@ namespace {
 		}
 	};
 
+	/// Specialization
 	template<bool IsFloat>
 	struct ascii_strton_impl<true, IsFloat> {
 		template<typename T> inline
@@ -497,7 +525,8 @@ namespace {
 
 
 
-// Note: Parameter "base" is ignored for floating point types.
+/// Create any number (integral or floating point) from a string.
+/// Note: Parameter "base" is ignored for floating point types.
 template<typename T> inline
 T ascii_strton(const char* nptr, char** endptr, int base = 0)
 {
@@ -511,6 +540,11 @@ T ascii_strton(const char* nptr, char** endptr, int base = 0)
 }  // ns
 
 
+#ifdef _MSC_VER
+	#pragma warning(pop)
+#endif
 
 
 #endif
+
+/// @}
