@@ -1,8 +1,13 @@
 /**************************************************************************
  Copyright:
-      (C) 2009  Alexander Shaduri <ashaduri 'at' gmail.com>
+      (C) 2009 - 2012  Alexander Shaduri <ashaduri 'at' gmail.com>
  License: See LICENSE_zlib.txt file
 ***************************************************************************/
+/// \file
+/// \author Alexander Shaduri
+/// \ingroup applib
+/// \weakgroup applib
+/// @{
 
 #ifndef HZ_CSTDINT_H
 #define HZ_CSTDINT_H
@@ -14,7 +19,7 @@
 // It contains all the C99 stdint.h stuff in global namespace.
 
 
-// C++0x defines these automatically for cstdint, so we do it too for compatibility.
+// C++11 defines these automatically for cstdint, so we do it too for compatibility.
 #ifndef __STDC_LIMIT_MACROS
 	#define __STDC_LIMIT_MACROS
 #endif
@@ -38,3 +43,5 @@
 
 
 #endif
+
+/// @}
