@@ -1,8 +1,13 @@
 /**************************************************************************
  Copyright:
-      (C) 2008 - 2009  Alexander Shaduri <ashaduri 'at' gmail.com>
+      (C) 2008 - 2012  Alexander Shaduri <ashaduri 'at' gmail.com>
  License: See LICENSE_zlib.txt file
 ***************************************************************************/
+/// \file
+/// \author Alexander Shaduri
+/// \ingroup hz
+/// \weakgroup hz
+/// @{
 
 #ifndef HZ_FS_TOOLS_H
 #define HZ_FS_TOOLS_H
@@ -14,6 +19,7 @@
 
 #ifdef _WIN32
 	#include <direct.h>  // _wgetcwd, _wchdir
+	#include <string.h>  // free
 #else
 	#include <unistd.h>  // getcwd, chdir
 #endif
@@ -26,30 +32,33 @@
 #endif
 
 
-// Filesystem utilities
+/**
+\file
+Filesystem utilities
+*/
 
 
 namespace hz {
 
 
 
-// Get the current user's configuration file directory (in native fs encoding
-// for UNIX, utf-8 for windows).
+/// Get the current user's configuration file directory (in native fs encoding
+/// for UNIX, utf-8 for windows). E.g. "$HOME/.config" in UNIX.
 inline std::string get_user_config_dir();
 
-// Get the current user's home directory (in native fs encoding for UNIX,
-// utf-8 for windows). This function always returns something.
-// Note that the directory may not actually exist at all.
+/// Get the current user's home directory (in native fs encoding for UNIX,
+/// utf-8 for windows). This function always returns something, but
+/// note that the directory may not actually exist at all.
 inline std::string get_home_dir();
 
-// Get current working directory
+/// Get current working directory
 inline std::string get_current_dir();
 
-// Change current directory.
+/// Change current directory.
 inline bool set_current_dir(const std::string& dir);
 
-// Get temporary directory of a system (can be user-specific).
-// For windows it seems to be in UTF-8, for others - in fs encoding.
+/// Get temporary directory of a system (can be user-specific).
+/// For windows it seems to be in UTF-8, for others - in fs encoding.
 inline std::string get_tmp_dir();
 
 
@@ -59,8 +68,6 @@ inline std::string get_tmp_dir();
 
 
 
-// Get the current user's configuration file directory (in native fs encoding
-// for UNIX, utf-8 for windows).
 inline std::string get_user_config_dir()
 {
 	std::string dir;
@@ -82,10 +89,6 @@ inline std::string get_user_config_dir()
 
 
 
-
-// Get the current user's home directory (in native fs encoding for UNIX,
-// utf-8 for windows). This function always returns something.
-// Note that the directory may not actually exist at all.
 inline std::string get_home_dir()
 {
 	// Do NOT use g_get_home_dir, it doesn't work consistently in win32
@@ -125,22 +128,20 @@ inline std::string get_home_dir()
 
 
 
-// Get current working directory
 inline std::string get_current_dir()
 {
 	std::string dir;
 
 #ifdef _WIN32
-	std::size_t buf_size = MAX_PATH;
-	wchar_t* buf = new wchar_t[buf_size];  // not sure if it's really max, so increase it on demand
-
-	while (_wgetcwd(buf, static_cast<int>(buf_size)) == 0) {
-		delete[] buf;
-		buf = new wchar_t[buf_size *= 2];
+	wchar_t* buf = 0;
+	if (!(buf = _wgetcwd(NULL, 0))) {  // allocates buf with malloc()
+		return std::string();  // error
 	}
 
 	dir = hz::win32_utf16_to_utf8_string(buf);
 
+	free(buf);
+
 #else
 	std::size_t buf_size = 1024;
 	char* buf = new char[buf_size];
@@ -151,16 +152,15 @@ inline std::string get_current_dir()
 	}
 
 	dir = buf;
-#endif
 
 	delete[] buf;
+#endif
 
 	return dir;
 }
 
 
 
-// Change current directory.
 inline bool set_current_dir(const std::string& dir)
 {
 #ifdef _WIN32
@@ -173,8 +173,6 @@ inline bool set_current_dir(const std::string& dir)
 
 
 
-// Get temporary directory of a system (can be user-specific).
-// For windows it's in UTF-8, for others - in fs encoding.
 inline std::string get_tmp_dir()
 {
 	std::string dir;
@@ -212,3 +210,5 @@ inline std::string get_tmp_dir()
 
 
 #endif
+
+/// @}
