@@ -1,8 +1,13 @@
 /**************************************************************************
  Copyright:
-      (C) 2008 - 2009  Alexander Shaduri <ashaduri 'at' gmail.com>
+      (C) 2008 - 2012  Alexander Shaduri <ashaduri 'at' gmail.com>
  License: See LICENSE_zlib.txt file
 ***************************************************************************/
+/// \file
+/// \author Alexander Shaduri
+/// \ingroup hz
+/// \weakgroup hz
+/// @{
 
 #ifndef HZ_STRING_NUM_H
 #define HZ_STRING_NUM_H
@@ -20,45 +25,56 @@
 
 #include "type_properties.h"  // type_is_*
 #include "type_categories.h"  // type_check_category
-#include "static_assert.h"  // HZ_STATIC_ASSERT
+// #include "static_assert.h"  // HZ_STATIC_ASSERT
 #include "ascii.h"  // ascii_*
 
 
-
-// String to number and number to string conversions
+/**
+\file
+String to number and number to string conversions
+*/
 
 
 
 namespace hz {
 
 
-	// Check whether a string represents a numeric value (the value must
-	// be represented in C locale).
+	/// Check whether a string represents a numeric value (the value must
+	/// be represented in C locale).
 
-	// strict == true indicates that the whole string must represent a number exactly.
-	// strict == false allows the string to contain the number only in its beginning
-	// (ignores any leading spaces and trailing garbage).
-	// base has an effect only for integral types. For bool it specifies whether to
-	// accept "true" and "false" (as opposed to 1 and 0). For others, base should
-	// be between 2 and 36 inclusive.
-
-	// This function has no definition, only specializations.
+	/// strict == true indicates that the whole string must represent a number exactly.
+	/// strict == false allows the string to contain the number only in its beginning
+	/// (ignores any leading spaces and trailing garbage).
+	/// base has an effect only for integral types. For bool it specifies whether to
+	/// accept "true" and "false" (as opposed to 1 and 0). For others, base should
+	/// be between 2 and 36 inclusive.
+	/// This function has no definition, only specializations.
 	template<typename T> inline
 	bool string_is_numeric(const std::string& s, T& number, bool strict, int base_or_boolalpha);
 
-	// Short version with default base. (Needed because default base is different for bool and int).
+	/// Short version with default base. (Needed because default base is different for bool and int).
 	template<typename T> inline
 	bool string_is_numeric(const std::string& s, T& number, bool strict = true);
 
 
-	// Convert numeric value to string. alpha_or_base_or_precision means:
-	// for bool, 0 means 1/0, 1 means true/false;
-	// for int family (including char), it's the base to format in (8, 10, 16 are definitely supported);
-	// for float family, it controls the number of digits after comma.
+	/// A convenience string_is_numeric wrapper.
+	/// Note that in strict mode, T() is returned for invalid values.
 	template<typename T> inline
-	std::string number_to_string(T number, int alpha_or_base_or_precision);
+	T string_to_number(const std::string& s, bool strict, int base_or_boolalpha);
 
-	// Short version with default base / precision.
+	/// Short version with default base. (Needed because default base is different for bool and int).
+	template<typename T> inline
+	T string_to_number(const std::string& s, bool strict = true);
+
+
+	/// Convert numeric value to string. alpha_or_base_or_precision means:
+	/// for bool, 0 means 1/0, 1 means true/false;
+	/// for int family (including char), it's the base to format in (8, 10, 16 are definitely supported);
+	/// for float family, it controls the number of digits after comma.
+	template<typename T> inline
+	std::string number_to_string(T number, int alpha_or_base_or_precision, bool fixed_prec = false);
+
+	/// Short version with default base / precision.
 	template<typename T> inline
 	std::string number_to_string(T number);
 
@@ -76,6 +92,26 @@ namespace hz {
 
 
 
+template<typename T> inline
+T string_to_number(const std::string& s, bool strict, int base_or_boolalpha)
+{
+	T value = T();
+	hz::string_is_numeric(s, value, strict, base_or_boolalpha);
+	return value;
+}
+
+
+
+template<typename T> inline
+T string_to_number(const std::string& s, bool strict)
+{
+	T value = T();
+	hz::string_is_numeric(s, value, strict);
+	return value;
+}
+
+
+
 // Definition of the one without base parameter.
 template<typename T> inline
 bool string_is_numeric(const std::string& s, T& number, bool strict)
@@ -284,9 +320,9 @@ namespace internal {
 
 // public function with 2 parameters
 template<typename T> inline
-std::string number_to_string(T number, int boolalpha_or_base_or_precision)
+std::string number_to_string(T number, int boolalpha_or_base_or_precision, bool fixed_prec)
 {
-	return internal::number_to_string_impl<T>::func(number, boolalpha_or_base_or_precision, true);
+	return internal::number_to_string_impl<T>::func(number, boolalpha_or_base_or_precision, fixed_prec);
 }
 
 
@@ -321,3 +357,5 @@ std::string number_to_string(T number)
 
 
 #endif
+
+/// @}
