@@ -1,8 +1,13 @@
 /**************************************************************************
  Copyright:
-      (C) 2008 - 2009  Alexander Shaduri <ashaduri 'at' gmail.com>
+      (C) 2008 - 2012  Alexander Shaduri <ashaduri 'at' gmail.com>
  License: See LICENSE_zlib.txt file
 ***************************************************************************/
+/// \file
+/// \author Alexander Shaduri
+/// \ingroup hz
+/// \weakgroup hz
+/// @{
 
 #ifndef HZ_SYNC_H
 #define HZ_SYNC_H
@@ -18,7 +23,8 @@
 
 
 
-/*
+/**
+\file
 Threading policies providing synchronization primitives
 through wrapping existing thread libraries.
 
@@ -38,27 +44,27 @@ Every policy class provides:
 
 	Mutex Types:
 
-		* Mutex type. Possibly a wrapper around native mutex type.
+		- Mutex type. Possibly a wrapper around native mutex type.
 			May or may not be recursive. No guaranteed methods.
 			Guaranteed freeing of resource when destroyed.
-		* NativeMutex type. The underlying mutex type, if available.
+		- NativeMutex type. The underlying mutex type, if available.
 			If not available, it's typedefed as Mutex. No guaranteed
 			methods. Don't declare objects of this type - it is there solely
 			for interaction with existing native mutexes.
 
-		* RecMutex type. Same as Mutex, but recursive. No guaranteed
+		- RecMutex type. Same as Mutex, but recursive. No guaranteed
 			methods. Guaranteed freeing of resource when destroyed.
-		* NativeRecMutex type. Same as NativeMutex, but recursive.
+		- NativeRecMutex type. Same as NativeMutex, but recursive.
 			No guaranteed methods.
 
-		* RWMutex type. A mutex which supports read / write locking.
+		- RWMutex type. A mutex which supports read / write locking.
 			No guaranteed methods.
 			Guaranteed freeing of resource when destroyed.
 			Note: There is no recursion or lock escalation with RWLock.
 			That is, locking it multiple times (read or write), or write-locking
 			while holding a read-lock may work in some situations with some
 			backends, but can also cause a deadlock.
-		* NativeRWMutex type. A cross between NativeMutex and RWMutex.
+		- NativeRWMutex type. A cross between NativeMutex and RWMutex.
 
 		The Native types are needed to give an explicit guarantee that these
 		types are lockable via respective ScopedLock classes and static methods.
@@ -66,7 +72,7 @@ Every policy class provides:
 
 	Static Methods:
 
-		bool init()
+		bool init()\n
 			Initialize the thread backend. The application MUST call this function
 			prior to locking any mutexes from this policy. True is returned in case
 			the backend is initialized successfully, or has been initialized already.
@@ -83,24 +89,24 @@ Every policy class provides:
 		reason for this is that the system is already in unstable state, so
 		throwing a fatal exception seems to be the right choice.
 
-		void lock(MutexType& m)
+		void lock(MutexType& m)\n
 			Preconditions: m must not be locked by current thread.
 			Locks the mutex via the mutex's defined locking facilities.
 			This function will block until m is available for locking.
 
-		bool trylock(MutexType& m)
+		bool trylock(MutexType& m)\n
 			Preconditions: m must not be locked by current thread.
 			Tries to lock the mutex via the mutex's defined locking facilities.
 			This function will return false if lock could not be obtained, and
 			true otherwise.
 
-		void unlock(MutexType& m)
+		void unlock(MutexType& m)\n
 			Preconditions: m must be locked by current thread.
 			Unlocks the previously locked mutex immediately.
 
-		void lock(MutexType& m, bool for_write = false)
-		bool trylock(MutexType& m, bool for_write = false)
-		void unlock(MutexType& m, bool for_write = false)
+		void lock(MutexType& m, bool for_write = false)\n
+		bool trylock(MutexType& m, bool for_write = false)\n
+		void unlock(MutexType& m, bool for_write = false)\n
 			Same as above, but for RWMutex and NativeRWMutex types.
 			The second parameter must be equal for any lock / unlock
 			and trylock / unlock pair. If not, the behaviour is undefined.
@@ -120,20 +126,20 @@ Every policy class provides:
 		In case of error, sync_resource_error is thrown. Note that this exception
 		may be thrown from constructor. In that case, destruction is still necessary.
 
-		template<typename MutexType>
-		class GenericScopedLock
-			Constructor:
-				GenericScopedLock(MutexType& mutex, bool do_lock = true)
+		template\<typename MutexType\>\n
+		class GenericScopedLock\n
+			Constructor:\n
+				GenericScopedLock(MutexType& mutex, bool do_lock = true)\n
 			Guaranteed to accept Mutex, NativeMutex, RecMutex,
 			NativeRecMutex types.
 			If do_lock is true, locks mutex when constructed and unlocks when
 			destroyed. If do_lock is false, no operations are performed.
 			Uses policy's static methods lock() and unlock() for doing it.
 
-		template<typename MutexType>
-		class GenericScopedTryLock
-			Constructor:
-				GenericScopedTryLock(MutexType& mutex, bool do_lock = true)
+		template\<typename MutexType\>\n
+		class GenericScopedTryLock\n
+			Constructor:\n
+				GenericScopedTryLock(MutexType& mutex, bool do_lock = true)\n
 			Guaranteed to accept Mutex, NativeMutex, RecMutex,
 			NativeRecMutex types.
 			Behaves like ScopedTryLock, but provides an additional
@@ -142,42 +148,42 @@ Every policy class provides:
 			the operation (the lock must not be obtained at that point yet).
 			Uses policy's static methods trylock() and unlock() for doing it.
 
-		class GenericScopedRecLock, class GenericScopedRecTryLock
+		class GenericScopedRecLock, class GenericScopedRecTryLock\n
 			There are no such types, use GenericScopedLock and
 			GenericScopedTryLock instead. They accept recursive mutex types
 			too. This is done to avoid requiring knowledge of the exact mutex
 			type (non-recursive or recursive, non-native or native) when locking them.
 
-		template<typename MutexType>
-		class GenericScopedRWLock
-			Constructor:
-				ScopedRWLock(MutexType& mutex, bool for_write = false, bool do_lock = true)
+		template\<typename MutexType\>\n
+		class GenericScopedRWLock\n
+			Constructor:\n
+				ScopedRWLock(MutexType& mutex, bool for_write = false, bool do_lock = true)\n
 			Guaranteed to accept RWMutex, NativeRWMutex types.
 			Behaves similarily to ScopedLock, but works with read / write
 			mutexes instead.
 
-		template<typename MutexType>
-		class GenericScopedRWTryLock
-			Constructor:
-				ScopedRWTryLock(MutexType& mutex, bool for_write = false, bool do_lock = true)
+		template\<typename MutexType\>\n
+		class GenericScopedRWTryLock\n
+			Constructor:\n
+				ScopedRWTryLock(MutexType& mutex, bool for_write = false, bool do_lock = true)\n
 			A cross between ScopedRWLock and ScopedTryLock.
 
 		ScopedLock, ScopedTryLock, ScopedNativeLock, ScopedNativeTryLock,
 		ScopedRecLock, ScopedRecTryLock, ScopedNativeRecLock, ScopedNativeRecTryLock,
-		ScopedRWLock, ScopedRWTryLock, ScopedNativeRWLock, ScopedNativeRWTryLock
+		ScopedRWLock, ScopedRWTryLock, ScopedNativeRWLock, ScopedNativeRWTryLock\n
 			These are typedefs for respective (mutex, generic scoped lock) pairs.
 
 
 Out-of-policy Class templates:
 
-	template<typename MutexType>
-	struct SyncGetPolicy { typedef unspecified_policy_type type; };
+	template\<typename MutexType\>\n
+	struct SyncGetPolicy { typedef unspecified_policy_type type; };\n
 		This struct provides a way to retrieve a policy class type by supplying
 		mutex type. Any mutex type is supported, provided that such mutex type
 		exists in a policy somewhere.
 		For example, to do a locking in a function template where the exact
 		policy or mutex type is unknown, one could write:
-
+\code
 		template<typename MutexType>
 		void f(MutexType& m)
 		{
@@ -185,16 +191,16 @@ Out-of-policy Class templates:
 			// ...
 			// lock is released at the end of scope.
 		}
-
+\endcode
 		This also provides safe defaults for class templates which accept mutexes
 		as their template parameters, for example:
-
+\code
 		template<typename MutexType,
 			typename Policy = typename SyncGetPolicy<MutexType>::type>
 		class C {
 			// use Policy::ScopedLock, etc...
 		};
-
+\endcode
 */
 
 
@@ -204,14 +210,17 @@ namespace hz {
 
 
 
-// This is thrown in case of really bad problems
+/// This is thrown in case of really bad problems
 struct sync_resource_error : virtual public std::exception {  // from <exception>
 
+	/// Constructor
 	sync_resource_error(const std::string& why) : why_(why)
 	{ }
 
+	/// Virtual destructor
 	virtual ~sync_resource_error() throw() { }
 
+	// Reimplemented from std::exception
  	virtual const char* what() const throw()
 	{
 		msg = "hz::sync_resource_error: " + why_;
@@ -220,8 +229,8 @@ struct sync_resource_error : virtual public std::exception {  // from <exception
 
 	// yes, they should not be strings, to avoid memory allocations.
 	// no, I don't know how to return const char* without memory allocations.
-	const std::string why_;
-	mutable std::string msg;  // This must be a member to avoid its destruction on function call return. use what().
+	const std::string why_;  ///< The reason.
+	mutable std::string msg;  ///< The message (used in implementation of what()). This must be a member to avoid its destruction on function call return. use what().
 };
 
 
@@ -231,8 +240,7 @@ struct sync_resource_error : virtual public std::exception {  // from <exception
 
 
 
-// These class provides commonly used classes for policies.
-
+// This class provides commonly used classes for policies.
 template<typename Policy>
 struct SyncScopedLockProvider {
 
@@ -396,15 +404,15 @@ struct SyncScopedLockProvider {
 
 
 
-// A type useful for dummy mutexes, etc...
-// TypeChanger may be used to generate different (non-inter-convertible)
-// types, for e.g. function overloading.
+/// A type useful for dummy mutexes, etc...
+/// TypeChanger may be used to generate different (non-inter-convertible)
+/// types, for e.g. function overloading.
 template<int TypeChanger>
 struct SyncEmptyType { };
 
 
 
-// Classes may use this in single-threaded or non-locking environments.
+/// Classes may use this in single-threaded or non-locking environments.
 struct SyncPolicyNone {
 	// We use different types here to avoid user errors such as mixing
 	// types when developing with single-threaded version.
@@ -443,7 +451,7 @@ struct SyncPolicyNone {
 	static void unlock(NativeRWMutex& m, bool for_write = false) { }
 
 
-	// Dummy scoped lock, does absolutely nothing. Works for all of them
+	/// Dummy scoped lock, does absolutely nothing. Works with all the mutex types.
 	template<typename MutexType>
 	class GenericScopedLock : public hz::noncopyable {
 		public:
@@ -625,3 +633,5 @@ template<> struct SyncGetPolicy<SyncPolicyNone::NativeRWMutex> { typedef SyncPol
 
 
 #endif
+
+/// @}
