@@ -1,8 +1,13 @@
 /**************************************************************************
  Copyright:
-      (C) 2008 - 2009  Alexander Shaduri <ashaduri 'at' gmail.com>
+      (C) 2008 - 2012  Alexander Shaduri <ashaduri 'at' gmail.com>
  License: See LICENSE_zlib.txt file
 ***************************************************************************/
+/// \file
+/// \author Alexander Shaduri
+/// \ingroup hz
+/// \weakgroup hz
+/// @{
 
 #ifndef HZ_SYNC_PART_GET_POLICY_H
 #define HZ_SYNC_PART_GET_POLICY_H
@@ -10,14 +15,16 @@
 #include "hz_config.h"  // feature macros
 
 
-// Note: This is an internal file. Do NOT include manually.
-
+/**
+\file
+Internal header, do not include manually.
+*/
 
 namespace hz {
 
 
-	// Get policy class by Mutex type.
-	// Policies should specialize this struct.
+	/// Get policy class by Mutex type.
+	/// Policies should specialize this struct.
 	template<class MutexType>
 	struct SyncGetPolicy;  // no definition, only specializations.
 
@@ -30,3 +37,5 @@ namespace hz {
 
 
 #endif
+
+/// @}
