@@ -1,8 +1,13 @@
 /**************************************************************************
  Copyright:
-      (C) 2008 - 2009  Alexander Shaduri <ashaduri 'at' gmail.com>
+      (C) 2008 - 2012  Alexander Shaduri <ashaduri 'at' gmail.com>
  License: See LICENSE_zlib.txt file
 ***************************************************************************/
+/// \file
+/// \author Alexander Shaduri
+/// \ingroup hz
+/// \weakgroup hz
+/// @{
 
 #ifndef HZ_TLS_POLICY_GLIB_H
 #define HZ_TLS_POLICY_GLIB_H
@@ -12,7 +17,10 @@
 #include <glib.h>
 
 
-// Glib-based TLS.
+/**
+\file
+Glib-based thread-local storage.
+*/
 
 
 namespace hz {
@@ -29,6 +37,57 @@ extern "C" {
 
 
 
+#if GLIB_CHECK_VERSION(2, 32, 0)
+
+
+/// Glib-based TLS policy
+class TlsPolicyGlib {
+	public:
+
+		TlsPolicyGlib(tls_policy_glib_cleanup_func_t native_cleanup)
+				: key_((GPrivate)G_PRIVATE_INIT(native_cleanup))
+		{ }
+
+#if defined HAVE_CXX_EXTERN_C_OVERLOAD && HAVE_CXX_EXTERN_C_OVERLOAD
+		TlsPolicyGlib(tls_policy_glib_cleanup_func_cpp_t native_cleanup)
+				: key_((GPrivate)G_PRIVATE_INIT(native_cleanup))
+		{ }
+#endif
+
+		~TlsPolicyGlib()
+		{
+			// Nothing
+		}
+
+		void* get() const
+		{
+			return g_private_get(&key_);
+		}
+
+		void reset(void* p)
+		{
+			// this will call the cleanup function for the previous pointer
+			g_private_replace(&key_, static_cast<gpointer>(p));
+		}
+
+
+		static const bool cleanup_supported = true;
+
+	private:
+
+		mutable GPrivate key_;
+
+		TlsPolicyGlib(const TlsPolicyGlib&);
+		TlsPolicyGlib& operator= (const TlsPolicyGlib& from);
+};
+
+
+
+#else  // old glib
+
+
+
+/// Glib-based TLS policy
 class TlsPolicyGlib {
 	public:
 
@@ -69,7 +128,7 @@ class TlsPolicyGlib {
 	private:
 
 		mutable GStaticPrivate key_;
-		tls_policy_glib_cleanup_func_t native_cleanup_;  // may be NULL
+		tls_policy_glib_cleanup_func_t native_cleanup_;  ///< may be NULL
 
 		TlsPolicyGlib(const TlsPolicyGlib&);
 		TlsPolicyGlib& operator= (const TlsPolicyGlib& from);
@@ -77,7 +136,7 @@ class TlsPolicyGlib {
 
 
 
-
+#endif
 
 
 
@@ -88,3 +147,5 @@ class TlsPolicyGlib {
 
 
 #endif
+
+/// @}
